package com.jzlife.report.controller;

import java.time.Duration;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson.JSONObject;
import com.jzlife.report.service.redis.RedisService;
import com.jzlife.report.service.report.BigScreeReportService;
import com.jzlife.report.util.DateUtil;
import com.jzlife.report.util.DateUtils;
import com.jzlife.report.util.ResponseUtil;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

@RestController
@RequestMapping("bigScreen")
@Api(value = "BigScreenReportController", description = "大屏报表")
public class BigScreenReportController {

	@Autowired
	BigScreeReportService bigScreeReportService;

	@Autowired
	RedisService redisService;

	// 按日切换报表，
	@ApiOperation("年度鲸致派单--预约单")
	@GetMapping("yearServiceGMV")
	public Object yearServiceGMV(String date, Integer type) {
		if (StringUtils.isBlank(date)) {
			return ResponseUtil.fail("请选择日期");
		}
		String key = "bigreport:yearBookedGMV:" + date + type;
		String val = redisService.get(key);
		if (val != null) {
			return JSONObject.parseObject(val);
		}

		//缓存1小时
		Object yearServiceGMV = bigScreeReportService.yearServiceGMV(date, type);
		Duration duration = Duration.between(LocalDateTime.now(), LocalDateTime.now().plusMinutes(60));
		redisService.set(key, JSONObject.toJSONString(yearServiceGMV), duration);
		return yearServiceGMV;
	}

	// 实时销售明细,日总销售GMV，地图数据 30秒刷新
	@ApiOperation("实时销售明细,日总销售GMV，地图数据 30秒刷新--此接口 弃用")
	@GetMapping("salesGmvAndBookcount")
	public Object salesGmvAndBookcount(String date) {
		if (StringUtils.isBlank(date)) {
			return ResponseUtil.fail("请选择日期");
		}
		String key = "bigreport:salesGmvAndBookcount:" + date;
		String val = redisService.get(key);
		if (val != null) {
			return JSONObject.parseObject(val);
		}

		Object salesGmvAndBookcount = bigScreeReportService.salesGmvAndBookcount(date);

		Duration duration = Duration.between(LocalDateTime.now(), LocalDateTime.now().plusSeconds(28));
		redisService.set(key, JSONObject.toJSONString(salesGmvAndBookcount), duration);
		return salesGmvAndBookcount;
	}

	@ApiOperation("月累计gmv&派单量&地图数据 统一接口返回")
	@GetMapping("monthGmv")
	public Object monthGmv(String date) {
		if (StringUtils.isBlank(date)) {
			return ResponseUtil.fail("请选择日期");
		}
		String key = "bigreport:monthGmv:" + date;
		String val = redisService.get(key);
		if (val != null) {
			return JSONObject.parseObject(val);
		}

		Object monthGmv = bigScreeReportService.monthGmv(date);

		// 缓存28秒 因为前端是30秒刷新一次
		Duration duration = Duration.between(LocalDateTime.now(), LocalDateTime.now().plusSeconds(28));
		redisService.set(key, JSONObject.toJSONString(monthGmv), duration);
		return monthGmv;
	}


}
