package com.jzlife.report.controller;

import com.jzlife.report.anno.LogRecordFiled;
import com.jzlife.report.entity.reports.*;
import com.jzlife.report.service.report.BjManagerToolService;
import com.jzlife.report.util.ExcelUtils;
import com.jzlife.report.util.ResponseUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping("bjManagerTool")
@Api(value = "BjManagerTool-Controller", description = "保洁管理工具")
public class BjManagerToolController {

    @Autowired
    private BjManagerToolService bjManagerToolService;


    @ApiOperation(value = "可邀约好评-服务站信息", httpMethod = "GET", notes = "时间格式为：yyyy-MM")
    @GetMapping("evaluationInfo")
    @LogRecordFiled(funName = "可邀约好评")
    public Object evaluationInfo(HttpServletRequest request, @RequestParam(value = "citys", required = false) String citys, @RequestParam(required = false) String month, @RequestParam(required = false) String station) {
        if (StringUtils.isEmpty(citys)) {
            citys = request.getHeader("allowCity");
        }
        if (null == month) {
            month = LocalDate.now().format(DateTimeFormatter.ISO_LOCAL_DATE).substring(0, 7);
        }
        Map<String, Object> result = bjManagerToolService.evaluationInfo(citys, month, station);
        return ResponseUtil.ok(result);
    }


    @ApiOperation(value = "可邀约好评-服务站明细", httpMethod = "GET", notes = "时间格式为：yyyy-MM")
    @GetMapping("evaluationInfoByStation")
    @LogRecordFiled(funName = "可邀约好评-服务站明细")
    public Object evaluationInfoByStation(HttpServletRequest request, @RequestParam(value = "citys", required = false) String citys, @RequestParam(required = false) String month, @RequestParam(required = false) String station, @RequestParam(required = false) Integer pageNum, @RequestParam(required = false) Integer pageSize, Boolean export, HttpServletResponse response) throws Exception {
        if (StringUtils.isEmpty(citys)) {
            citys = request.getHeader("allowCity");
        }
        if (null == month) {
            month = LocalDate.now().format(DateTimeFormatter.ISO_LOCAL_DATE).substring(0, 7);
        }
        if (pageNum == null) {
            pageNum = 1;
        }
        if (pageSize == null) {
            pageSize = 50;
        }
        if (null != export && export) {
            List<EvaluationInfoDetailResult> noPageResult = bjManagerToolService.evaluationInfoByStationNoPage(citys, month, station, pageNum, pageSize);
            ExcelUtils.exportExcel(noPageResult, "可邀约好评-服务站每日明细", "可邀约好评-服务站每日明细-" + month, EvaluationInfoDetailResult.class, "可邀约好评-服务站每日明细-" + month, response);
            return null;
        }
        Map<String, Object> result = bjManagerToolService.evaluationInfoByStation(citys, month, station, pageNum, pageSize);
        return ResponseUtil.ok(result);
    }


    @ApiOperation(value = "可邀约好评-个人明细", httpMethod = "GET", notes = "时间格式为：yyyy-MM")
    @GetMapping("evaluationInfoByPerson")
    @LogRecordFiled(funName = "可邀约好评-个人明细")
    public Object evaluationInfoByPerson(HttpServletRequest request, @RequestParam(value = "citys", required = false) String citys, @RequestParam(required = false) String month, @RequestParam(required = false) String station, @RequestParam(required = false) Integer pageNum, @RequestParam(required = false) Integer pageSize, Boolean export, HttpServletResponse response) throws Exception {
        if (StringUtils.isEmpty(citys)) {
            citys = request.getHeader("allowCity");
        }
        if (null == month) {
            month = LocalDate.now().format(DateTimeFormatter.ISO_LOCAL_DATE).substring(0, 7);
        }
        if (pageNum == null) {
            pageNum = 1;
        }
        if (pageSize == null) {
            pageSize = 50;
        }
        if (null != export && export) {
            List<EvaluationInfoDetailByPersonResult> noPageResult = bjManagerToolService.evaluationInfoByPearsonNoPage(citys, month, station, pageNum, pageSize);
            ExcelUtils.exportExcel(noPageResult, "可邀约好评-个人明细", "可邀约好评-个人明细-" + month, EvaluationInfoDetailByPersonResult.class, "可邀约好评-个人明细-" + month, response);
            return null;
        }
        Map<String, Object> result = bjManagerToolService.evaluationInfoByPearson(citys, month, station, pageNum, pageSize);
        return ResponseUtil.ok(result);
    }

    @ApiOperation(value = "个人排名", httpMethod = "GET", notes = "时间格式为：yyyy-MM")
    @GetMapping("individualRanking")
    @LogRecordFiled(funName = "个人排名")
    public Object individualRanking(HttpServletRequest request,
                                    @RequestParam(value = "citys", required = false) String citys,
                                    @RequestParam(value = "station", required = false) String stations,
                                    @RequestParam(required = false) String month,
                                    @RequestParam(required = false) String workerName,
                                    @RequestParam(required = false) Integer pageNum,
                                    @RequestParam(required = false) Integer pageSize, Boolean export,
                                    @RequestParam(required = false) String goodRank, HttpServletResponse response) throws Exception {

        //goodRank 好评率
        if (null == month) {
            month = LocalDate.now().format(DateTimeFormatter.ISO_LOCAL_DATE).substring(0, 7);
        }
        if (StringUtils.isEmpty(citys) && StringUtils.isEmpty(stations)) {
            citys = request.getHeader("allowCity");
        }
        if (pageNum == null) {
            pageNum = 1;
        }
        if (pageSize == null) {
            pageSize = 50;
        }
        if (null != export && export) {
            List<IndividualRankingResult> noPageResult = bjManagerToolService.individualRankingNoPage(citys, month, workerName, pageNum, pageSize, stations, goodRank);
            ExcelUtils.exportExcel(noPageResult, "个人排名", "个人排名-" + month, IndividualRankingResult.class, "个人排名-" + month, response);
            return null;
        }
        Map<String, Object> result = bjManagerToolService.individualRanking(citys, month, workerName, pageNum, pageSize, stations, goodRank);
        return ResponseUtil.ok(result);
    }

    @ApiOperation(value = "个人排名-个人明细", httpMethod = "GET", notes = "时间格式为：yyyy-MM")
    @GetMapping("individualRankingDetail")
    @LogRecordFiled(funName = "个人排名-个人明细")
    public Object individualRankingDetail(@RequestParam(value = "citys", required = false) String citys, @RequestParam(value = "station", required = false) String stations, @RequestParam(required = false) String month, @RequestParam(required = false) String workerName, @RequestParam(required = false) Integer pageNum, @RequestParam(required = false) Integer pageSize, Boolean export, HttpServletResponse response) throws Exception {
        if (null == month) {
            month = LocalDate.now().format(DateTimeFormatter.ISO_LOCAL_DATE).substring(0, 7);
        }
        if (pageNum == null) {
            pageNum = 1;
        }
        if (pageSize == null) {
            pageSize = 50;
        }
        if (null != export && export) {
            List<IndividualRankingDetailResult> noPageResult = bjManagerToolService.individualRankingDetailNoPage(month, workerName,citys,stations);
            ExcelUtils.exportExcel(noPageResult, "个人排名-个人明细", "个人排名-个人明细-" + month, IndividualRankingDetailResult.class, "个人排名-个人明细-" + month, response);
            return null;
        }
        Map<String, Object> result = bjManagerToolService.individualRankingDetail(month, workerName, pageNum, pageSize,citys,stations);
        return ResponseUtil.ok(result);
    }


    @ApiOperation(value = "签到打卡-城市维度汇总", httpMethod = "GET", notes = "时间格式为：yyyy-MM")
    @GetMapping("signAdnClockInfo")
    @LogRecordFiled(funName = "签到打卡-城市维度汇总")
    public Object signAdnClockInfo(HttpServletRequest request, @RequestParam(required = false) String citys, @RequestParam(required = false) String month) {
        if (StringUtils.isEmpty(citys)) {
            citys = request.getHeader("allowCity");
        }
        if (null == month) {
            month = LocalDate.now().format(DateTimeFormatter.ISO_LOCAL_DATE).substring(0, 7);
        }

        Map<String, Object> result = bjManagerToolService.signAdnClockInfo(citys, month);
        return ResponseUtil.ok(result);
    }

    @ApiOperation(value = "签到打卡-服务站维度汇总", httpMethod = "GET", notes = "时间格式为：yyyy-MM")
    @GetMapping("signAdnClockInfoByStation")
    @LogRecordFiled(funName = "签到打卡-服务站维度汇总")
    public Object signAdnClockInfoByStation(HttpServletRequest request, @RequestParam(required = false) String citys, @RequestParam(required = false) String month, @RequestParam(required = false) String station) {
        if (StringUtils.isEmpty(citys)) {
            citys = request.getHeader("allowCity");
        }
        if (null == month) {
            month = LocalDate.now().format(DateTimeFormatter.ISO_LOCAL_DATE).substring(0, 7);
        }

        Map<String, Object> result = bjManagerToolService.signAdnClockInfoByStation(citys, month, station);
        return ResponseUtil.ok(result);
    }

    @ApiOperation(value = "签到打卡-商家到门和离门明细", httpMethod = "GET", notes = "时间格式为：yyyy-MM,主要是查询迟到和过早到的明细")
    @GetMapping("signAdnClockOnDoorAndLeaveDoorInfo")
    @LogRecordFiled(funName = "签到打卡-商家到门和离门明细")
    public Object signAdnClockOnDoorAndLeaveDoorInfo(HttpServletRequest request,
                                                     @RequestParam(required = false) String citys,
                                                     @RequestParam(required = false) String month,
                                                     @RequestParam(required = false) String station,
                                                     @RequestParam(required = false) String workerName,
                                                     @RequestParam(value = "queryType") @ApiParam(name = "queryType", value = "1 到门 2离门") String queryType,
                                                     @RequestParam(required = false) String onOrLeaveDoorType,
                                                     @RequestParam(required = false) Integer pageNum,
                                                     @RequestParam(required = false) Integer pageSize,
                                                     Boolean export, HttpServletResponse response) throws Exception {
        if (StringUtils.isEmpty(citys)) {
            citys = request.getHeader("allowCity");
        }
        if (null == month) {
            month = LocalDate.now().format(DateTimeFormatter.ISO_LOCAL_DATE).substring(0, 7);
        }
        if (pageNum == null) {
            pageNum = 1;
        }
        if (pageSize == null) {
            pageSize = 50;
        }
        if (null != export && export) {
            List<SignAdnClockOnDoorInfoResult> noPageResult = bjManagerToolService.signAdnClockOnDoorAndLeaveDoorInfoNoPage(citys, month, station, workerName, queryType,onOrLeaveDoorType);
            ExcelUtils.exportExcel(noPageResult, "签到打卡-个人明细", "签到打卡-个人明细-" + month, SignAdnClockOnDoorInfoResult.class, "签到打卡-个人明细-" + month, response);
            return null;
        }
        Map<String, Object> result = bjManagerToolService.signAdnClockOnDoorAndLeaveDoorInfo(citys, month, station, workerName, queryType, pageNum, pageSize,onOrLeaveDoorType);
        return ResponseUtil.ok(result);
    }


}
