package com.jzlife.report.controller;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.jzlife.report.anno.LogRecordFiled;
import com.jzlife.report.dto.*;
import com.jzlife.report.dto.douyin.DouyinTotalVo;
import com.jzlife.report.service.report.CPService;
import com.jzlife.report.util.ExcelUtils;
import com.jzlife.report.util.ResponseUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.List;

@Slf4j
@Api(value = "企微相关", description = "企微相关的报表")
@RestController
@RequestMapping("cpReprot")
public class CPReportController {

    @Autowired
    private CPService cpService;

    @ApiOperation("预采任务拨打情况")
    @GetMapping("preCollect")
    @LogRecordFiled(funName = "预采任务拨打情况")
    public Object preCollect(
            @RequestParam(required = true) String startTime,
            @RequestParam(required = true) String endTime,
            @RequestParam(required = false) Integer campaignId,
            @RequestParam(defaultValue = "1") Integer pageIndex,
            @RequestParam(defaultValue = "50") Integer pageSize,
            @RequestParam(defaultValue = "false") Boolean isExport,
            HttpServletResponse response, HttpServletRequest request) throws Exception {
        if (StringUtils.isBlank(startTime) || StringUtils.isBlank(endTime)) {
            return ResponseUtil.fail("任务日期不能为空");
        }

        if (isExport == false)
            return ResponseUtil.ok(cpService.reportPreCollect(startTime, endTime, campaignId, pageIndex, pageSize));
        else {
            List<CPPreCollectVo> list = (List<CPPreCollectVo>) cpService.reportPreCollect(startTime, endTime, campaignId, pageIndex, pageSize);
            ExcelUtils.exportExcel(list, null, "Sheet1", CPPreCollectVo.class, "预采任务拨打情况.xls", response);
            return null;
        }
    }


    @ApiOperation("日常回访添加企微数量")
    @GetMapping("cpDailyCall")
    @LogRecordFiled(funName = "日常回访添加企微数量")
    public Object cpDailyCall(
            @RequestParam(required = true) String startTime,
            @RequestParam(required = true) String endTime,
            @RequestParam(defaultValue = "1") Integer pageIndex,
            @RequestParam(defaultValue = "50") Integer pageSize,
            @RequestParam(defaultValue = "false") Boolean isExport,
            HttpServletResponse response, HttpServletRequest request) throws Exception {
        if (StringUtils.isBlank(startTime) || StringUtils.isBlank(endTime)) {
            return ResponseUtil.fail("添加日期不能为空");
        }
        if (isExport == false) {
            return ResponseUtil.ok(cpService.reportCPDailyCall(startTime, endTime, pageIndex, pageSize));
        } else {
            pageIndex = 1;
            pageSize = 10000;
            List<CPDailyCallVo> list = cpService.reportCPDailyCall(startTime, endTime, pageIndex, pageSize);
            ExcelUtils.exportExcel(list, null, "Sheet1", CPDailyCallVo.class, "日常回访添加企微数量.xls", response);
            return null;
        }


    }

    @ApiOperation("员工企微添加数量")
    @GetMapping("cpJoin")
    @LogRecordFiled(funName = "员工企微添加数量")
    public Object cpJoin(
            @RequestParam(required = true) String startTime,
            @RequestParam(required = true) String endTime,
            @RequestParam(defaultValue = "false") Boolean isExport,
            HttpServletResponse response, HttpServletRequest request) throws Exception {
        if (StringUtils.isBlank(startTime) || StringUtils.isBlank(endTime)) {
            return ResponseUtil.fail("添加日期不能为空");
        }

        return ResponseUtil.ok(cpService.reportCPJoin(startTime, endTime));
    }


    @ApiOperation("企微添加数汇总")
    @GetMapping("cpTotal")
    @LogRecordFiled(funName = "企微添加数汇总")
    public Object cpTotal(
            @RequestParam(required = true) String startTime,
            @RequestParam(required = true) String endTime,
            @RequestParam(defaultValue = "1") Integer pageIndex,
            @RequestParam(defaultValue = "50") Integer pageSize,
            @RequestParam(defaultValue = "false") Boolean isExport,
            HttpServletResponse response, HttpServletRequest request) throws Exception {
        if (StringUtils.isBlank(startTime) || StringUtils.isBlank(endTime)) {
            return ResponseUtil.fail("添加日期不能为空");
        }

        if (isExport == null || isExport == false) {
            PageHelper.startPage(pageIndex, pageSize);
            List<CPTotalVo> list = cpService.selectCPTotal(startTime, endTime, pageIndex, pageSize);
            return ResponseUtil.ok(new PageInfo<>(list));
        } else {
            List<CPTotalVo> list = cpService.selectCPTotal(startTime, endTime, pageIndex, pageSize);
            ExcelUtils.exportExcel(list, null, "Sheet1", CPTotalVo.class, "CPTotalVo.xls", response);
            return null;

        }

    }

    @ApiOperation("企微添加数明细")
    @GetMapping("cpDetail")
    @LogRecordFiled(funName = "企微添加数明细")
    public Object cpDetail(
            @RequestParam(required = true) String startTime,
            @RequestParam(required = true) String endTime,
            @RequestParam(defaultValue = "1") Integer pageIndex,
            @RequestParam(defaultValue = "50") Integer pageSize,
            @RequestParam(defaultValue = "false") Boolean isExport,
            HttpServletResponse response, HttpServletRequest request) throws Exception {
        if (StringUtils.isBlank(startTime) || StringUtils.isBlank(endTime)) {
            return ResponseUtil.fail("添加日期不能为空");
        }

        if (isExport == null || isExport == false) {
            PageHelper.startPage(pageIndex, pageSize);
            List<CPDetailVo> list = cpService.selectCPDetail(startTime, endTime, pageIndex, pageSize);
            return ResponseUtil.ok(new PageInfo<>(list));
        } else {
            List<CPDetailVo> list = cpService.selectCPDetail(startTime, endTime, pageIndex, pageSize);
            ExcelUtils.exportExcel(list, null, "Sheet1", CPDetailVo.class, "CPDetailVo.xls", response);
            return null;

        }

    }
}

