package com.jzlife.report.controller;

import com.jzlife.report.anno.LogRecordFiled;
import com.jzlife.report.excelExport.*;
import com.jzlife.report.service.report.CleaningDataMonitorService;
import com.jzlife.report.util.ExcelUtils;
import com.jzlife.report.util.ResponseUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.math.BigDecimal;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping("cleaningDataMonitor")
@Api(value = "cleanDataMonitor-Controller", description = "保洁数据监控")
public class CleaningDataMonitorController {


    @Autowired
    private CleaningDataMonitorService cleaningDataMonitorService;


    @ApiOperation(value = "各城市BD销售业绩及总业绩&合同维度数据", httpMethod = "POST", notes = "时间格式为：yyyy-MM")
    @GetMapping("salesPerformance")
    @LogRecordFiled(funName = "各城市BD销售业绩及总业绩&合同维度数据")
    public Object salesPerformance(HttpServletRequest request,
                                   @RequestParam(required = false) String month,
                                   @RequestParam(required = false) Integer type,
                                   @RequestParam(required = false) Boolean isExport,
                                   HttpServletResponse response

    ) throws Exception {
        String citys = request.getHeader("allowCity");
        if (null == month) {
            month = LocalDate.now().format(DateTimeFormatter.ISO_LOCAL_DATE).substring(0, 7);
        }
        List<Map<String, Object>> result = cleaningDataMonitorService.salesPerformance(citys, month);
        if (null != isExport && isExport) {
            type = type == null ? 1 : type;
            //导出
            if (type == 1) {
                //个人(导购)
                List<PersonGuide> list = new ArrayList<>();
                for (int i = 0; i < result.size() - 1; i++) {
                    List<HashMap<String, Object>> a = (ArrayList) result.get(i);
                    for (HashMap<String, Object> stringObjectMap : a) {
                        String queryType = (String) stringObjectMap.get("query_type");
                        if (queryType.equals("个人(导购)")) {
                            PersonGuide personGuide = new PersonGuide();
                            personGuide.setCity((String) stringObjectMap.get("city"));
                            personGuide.setNannyName((String) stringObjectMap.get("nanny_name"));
                            personGuide.setDayRealFee((BigDecimal) stringObjectMap.get("day_real_fee"));
                            personGuide.setMonthRealFee((BigDecimal) stringObjectMap.get("month_real_fee"));
                            personGuide.setMonthFee((BigDecimal) stringObjectMap.get("month_fee"));
                            personGuide.setMonthRefund((BigDecimal) stringObjectMap.get("month_refund"));
                            list.add(personGuide);
                        }
                    }
                }
                if (list.size() > 0) {
                    ExcelUtils.exportExcel(list, "保洁监控-个人导购", "个人导购-" + month, PersonGuide.class, "个人导购-" + month, response);
                }
            } else if (type == 2) {
                //城市(导购)
                List<CityGuide> list = new ArrayList<>();
                for (int i = 0; i < result.size() - 1; i++) {
                    List<HashMap<String, Object>> a = (ArrayList) result.get(i);
                    for (HashMap<String, Object> stringObjectMap : a) {
                        String queryType = (String) stringObjectMap.get("query_type");
                        if (queryType.equals("城市(导购)")) {
                            CityGuide personGuide = new CityGuide();
                            personGuide.setCity((String) stringObjectMap.get("city"));
                            personGuide.setDayRealFee((BigDecimal) stringObjectMap.get("day_real_fee"));
                            personGuide.setMonthRealFee((BigDecimal) stringObjectMap.get("month_real_fee"));
                            personGuide.setMonthFee((BigDecimal) stringObjectMap.get("month_fee"));
                            personGuide.setMonthRefund((BigDecimal) stringObjectMap.get("month_refund"));
                            list.add(personGuide);

                        }
                    }
                }
                if (list.size() > 0) {
                    ExcelUtils.exportExcel(list, "保洁监控-城市导购", "城市导购-" + month, CityGuide.class, "城市导购-" + month, response);
                }
            } else if (type == 3) {
                //城市(总体)销售
                List<CitySale> list = new ArrayList<>();
                for (int i = 0; i < result.size() - 1; i++) {
                    List<HashMap<String, Object>> a = (ArrayList) result.get(i);
                    for (HashMap<String, Object> stringObjectMap : a) {
                        String queryType = (String) stringObjectMap.get("query_type");
                        if (queryType.equals("城市(总体)")) {
                            CitySale personGuide = new CitySale();
                            personGuide.setCity((String) stringObjectMap.get("city"));
                            personGuide.setDayRealFee((BigDecimal) stringObjectMap.get("day_real_fee"));
                            personGuide.setMonthRealFee((BigDecimal) stringObjectMap.get("month_real_fee"));
                            personGuide.setMonthFee((BigDecimal) stringObjectMap.get("month_fee"));
                            personGuide.setMonthRefund((BigDecimal) stringObjectMap.get("month_refund"));
                            list.add(personGuide);
                        }
                    }
                }
                if (list.size() > 0) {
                    ExcelUtils.exportExcel(list, "保洁监控-城市销售", "城市销售-" + month, CitySale.class, "城市销售-" + month, response);
                }
            } else if (type == 4) {
                //保姆签单-城市维度
                List<SignBill> list = new ArrayList<>();
                for (int i = 0; i < result.size() - 1; i++) {
                    List<HashMap<String, Object>> a = (ArrayList) result.get(i);
                    for (HashMap<String, Object> stringObjectMap : a) {
                        String queryType = (String) stringObjectMap.get("query_type");
                        if (queryType.equals("保姆签单-城市维度")) {
                            SignBill signBill = new SignBill();
                            signBill.setCity((String) stringObjectMap.get("city"));
                            signBill.setValidClue((Integer) stringObjectMap.get("valid_clue"));
                            signBill.setInvalidClue((Integer) stringObjectMap.get("invalid_clue"));
                            signBill.setTodayAllocationClue((Integer) stringObjectMap.get("today_allocation_clue"));
                            signBill.setTodayContractNum((Integer) stringObjectMap.get("today_contract_num"));
                            signBill.setTodayContractAmount((BigDecimal) stringObjectMap.get("today_contract_amount"));
                            signBill.setMonthContractNum((Integer) stringObjectMap.get("month_contract_num"));
                            signBill.setMonthContractAmount((BigDecimal) stringObjectMap.get("month_contract_amount"));
                            signBill.setRefundAmount((BigDecimal) stringObjectMap.get("refund_amount"));
                            signBill.setConversionRate((String) stringObjectMap.get("conversion_rate"));
                            signBill.setCustomerPrice((BigDecimal) stringObjectMap.get("customer_price"));
                            signBill.setReferralContractNum((Integer) stringObjectMap.get("referral_contract_num"));
                            signBill.setReferralContractAmount((BigDecimal) stringObjectMap.get("referral_contract_amount"));
                            signBill.setReferralContractRate((String) stringObjectMap.get("referral_contract_rate"));
                            list.add(signBill);
                        }
                    }
                }
                if (list.size() > 0) {
                    ExcelUtils.exportExcel(list, "保姆签单-城市维度", "保姆签单-" + month, SignBill.class, "保姆签单-" + month, response);
                }
            } else if (type == 5) {
                //保姆签单-个人维度
                List<SignBillPerson> list = new ArrayList<>();
                for (int i = 0; i <= result.size() - 1; i++) {
                    List<HashMap<String, Object>> a = (ArrayList) result.get(i);
                    for (HashMap<String, Object> stringObjectMap : a) {
                        String queryType = (String) stringObjectMap.get("query_type");
                        if (queryType.equals("保姆签单-个人维度")) {
                            SignBillPerson info = new SignBillPerson();
                            info.setCity((String) stringObjectMap.get("city"));
                            info.setNannyName((String) stringObjectMap.get("nanny_name"));
                            info.setCurrentDayAllotNum((Integer) stringObjectMap.get("current_day_allot_num"));
                            info.setCurrentMonthAllotNum((Integer) stringObjectMap.get("current_month_allot_num"));
                            info.setOneYearAllotNum((Integer) stringObjectMap.get("one_year_allot_num"));
                            info.setCurrentDayContractNum((Integer) stringObjectMap.get("current_day_contract_num"));
                            info.setCurrentDayContractAmont((BigDecimal) stringObjectMap.get("current_day_contract_amont"));
                            info.setCurrentMonthContractNum((Integer) stringObjectMap.get("current_month_contract_num"));
//                            info.setCurrentMonthContractAmont((BigDecimal) stringObjectMap.get("current_month_contract_amont"));
                            info.setCurrentMonthRefundAmount((BigDecimal) stringObjectMap.get("current_month_refund_amount"));
                            info.setCurrentMoenthRealAmont((BigDecimal) stringObjectMap.get("current_moenth_real_amont"));
                            info.setCurrentMonthConversionRate((String) stringObjectMap.get("current_month_conversion_rate"));
                            info.setCurrentMonthCustomerPrice((BigDecimal) stringObjectMap.get("current_month_customer_price"));
                            info.setTransferContractNum((Integer) stringObjectMap.get("transfer_contract_num"));
//                            info.setTransferContractAmount((BigDecimal) stringObjectMap.get("transfer_contract_amount"));
//                            info.setReferRefundAmount((BigDecimal) stringObjectMap.get("refer_refund_amount"));
                            info.setReferRealAmount((BigDecimal) stringObjectMap.get("refer_real_amount"));
                            info.setTransferContractRate((String) stringObjectMap.get("transfer_contract_rate"));
                            list.add(info);
                        }
                    }
                }
                if (list.size() > 0) {
                    ExcelUtils.exportExcel(list, "保姆签单-个人维度", "个人保姆签单-" + month, SignBillPerson.class, "个人保姆签单-" + month, response);
                }
            }
            return  null;
        }

        return ResponseUtil.ok(result);
    }


}
