package com.jzlife.report.controller;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.jzlife.report.anno.LogRecordFiled;
import com.jzlife.report.dao.douyin.DouyinPageDataMapper;
import com.jzlife.report.dao.douyin.DouyinSalesMapper;
import com.jzlife.report.dao.reports.DyCouponMapper;
import com.jzlife.report.dto.douyin.*;
import com.jzlife.report.entity.douyin.AppPageLogData;
import com.jzlife.report.entity.douyin.DouyinPageData;
import com.jzlife.report.entity.reports.DyLaikeShopEvaltion;
import com.jzlife.report.entity.reports.DyLaikeShopThreeRate;
import com.jzlife.report.service.DouyinShopEvaltionService;
import com.jzlife.report.service.DouyinShopThreeRateService;
import com.jzlife.report.service.douyin.DouyinPageService;
import com.jzlife.report.util.DateUtil;
import com.jzlife.report.util.ExcelUtils;
import com.jzlife.report.util.ResponseUtil;
import io.swagger.annotations.ApiOperation;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 抖音相关报表
 */
@RestController
@RequestMapping("douyin")
public class DouYinController {

	@Autowired
	DouyinPageService douyinPageService;

	@Autowired
	DouyinSalesMapper douyinSalesMapper;

	@Autowired
	private DouyinShopEvaltionService douyinShopEvaltionService;

	@Autowired
	private DouyinShopThreeRateService douyinShopThreeRateService;

	@Autowired
	DyCouponMapper dyCouponMapper;

	@Autowired
	DouyinPageDataMapper douyinPageDataMapper;

	@ApiOperation(value = "抖音小程序页面数据分析", httpMethod = "GET", notes = "抖音小程序页面数据分析")
	@GetMapping("pageData")
	@LogRecordFiled(funName = "抖音小程序页面数据分析")
	public Object getPageList(@RequestParam("startDate") String startDate, @RequestParam("endDate") String endDate,
			Boolean isExport, Integer page, Integer limit, HttpServletResponse response) throws Exception {
		if (isExport == null || isExport == false) {
			PageHelper.startPage(page, limit);
			List<DouyinPageData> list = douyinPageService.getList(startDate, endDate);
			return ResponseUtil.ok(new PageInfo<>(list));
		} else {
			List<DouyinPageData> list = douyinPageService.getList(startDate, endDate);
			ExcelUtils.exportExcel(list, null, "Sheet1", DouyinPageData.class, "douyinpage.xls", response);
			return null;
		}
	}

	@ApiOperation(value = "抖音小程序实时销量", httpMethod = "GET", notes = "抖音小程序实时销量")
	@GetMapping("salesData")
	@LogRecordFiled(funName = "抖音小程序实时销量")
	public Object getSalesData(@RequestParam("startTime") String startTime, @RequestParam("endTime") String endTime,
			@RequestParam("provider") String provider, @RequestParam("orderStatus") String orderStatus, Integer goodsId,
			String goodsName, Boolean isExport, Integer page, Integer limit, String orderChannel,
			HttpServletResponse response) throws Exception {
		if (isExport == null || !isExport) {
			PageHelper.startPage(page, limit);
			List<DouyinSalesVo> list = douyinSalesMapper.getList(startTime, endTime, provider, orderStatus, goodsId,
					goodsName, orderChannel);
			return ResponseUtil.ok(new PageInfo<>(list));
		} else {
			List<DouyinSalesVo> list = douyinSalesMapper.getList(startTime, endTime, provider, orderStatus, goodsId,
					goodsName, orderChannel);
			ExcelUtils.exportExcel(list, null, "Sheet1", DouyinSalesVo.class, "douyinpage.xls", response);
			return null;
		}
	}

	@ApiOperation(value = "美团小程序刷单实时销量", httpMethod = "GET", notes = "美团小程序实时销量")
	@GetMapping("salesMTData")
	@LogRecordFiled(funName = "美团小程序实时销量")
	public Object salesMTData(@RequestParam("startTime") String startTime, @RequestParam("endTime") String endTime,
			String orderSn, String mobile, String serialNumber, Boolean isExport, Integer page, Integer limit,
			HttpServletResponse response) throws Exception {
		if (isExport == null || !isExport) {
			PageHelper.startPage(page, limit);
			List<DouyinSalesVo> list = douyinSalesMapper.getMTList(startTime, endTime, mobile, serialNumber, orderSn);
			return ResponseUtil.ok(new PageInfo<>(list));
		} else {
			List<DouyinSalesVo> list = douyinSalesMapper.getMTList(startTime, endTime, mobile, serialNumber, orderSn);
			ExcelUtils.exportExcel(list, null, "Sheet1", DouyinSalesVo.class, "douyinpage.xls", response);
			return null;
		}
	}

	@ApiOperation(value = "抖音小程序销量汇总", httpMethod = "GET", notes = "抖音小程序销量汇总")
	@GetMapping("salesTotal")
	@LogRecordFiled(funName = "抖音小程序销量汇总")
	public Object salesTotal(@RequestParam("startTime") String startTime, @RequestParam("endTime") String endTime,
			String city, Boolean isExport, Integer page, Integer limit, HttpServletResponse response) throws Exception {
		if (isExport == null || isExport == false) {
			PageHelper.startPage(page, limit);
			List<DouyinTotalVo> list = douyinSalesMapper.getTotalList(startTime, endTime, city);
			return ResponseUtil.ok(new PageInfo<>(list));
		} else {
			List<DouyinTotalVo> list = douyinSalesMapper.getTotalList(startTime, endTime, city);
			ExcelUtils.exportExcel(list, null, "Sheet1", DouyinTotalVo.class, "DouyinTotalVo.xls", response);
			return null;
		}
	}


	@ApiOperation(value = "抖音小程序销量汇总by poi", httpMethod = "GET", notes = "抖音小程序销量汇总by poi")
	@GetMapping("salesTotalByPoi")
	@LogRecordFiled(funName = "抖音小程序销量汇总by poi")
	public Object salesTotalByPoi(@RequestParam("startTime") String startTime,
								  @RequestParam("endTime") String endTime,
							 String city, Boolean isExport,
								  Integer page,
								  Integer limit,
								  HttpServletResponse response) throws Exception {
		if (isExport == null || isExport == false) {
			PageHelper.startPage(page, limit);
			List<DouyinSalesByPoiVo> list = douyinSalesMapper.getTotalListByPoi(startTime, endTime, city);
			return ResponseUtil.ok(new PageInfo<>(list));
		} else {
			List<DouyinSalesByPoiVo> list = douyinSalesMapper.getTotalListByPoi(startTime, endTime, city);
			ExcelUtils.exportExcel(list, null, "Sheet1", DouyinSalesByPoiVo.class, "DouyinTotalVo.xls", response);
			return null;
		}
	}

	@ApiOperation("获取订单渠道")
	@GetMapping("getChannel")
	public Object getChannel() {
		List<Map<String, Object>> list = douyinSalesMapper.getChannel();
		return ResponseUtil.ok(list);
	}

	@ApiOperation("查询抖音门店评价数据")
	@GetMapping("getStoreComment")
	public Object getStoreComment(@RequestParam(value = "fraName", required = false) String fraName,
			@RequestParam(value = "shopName", required = false) String shopName,
			@RequestParam(value = "poiId", required = false) String poiId, @RequestParam("page") Integer page,
			@RequestParam("limit") Integer limit, Boolean isExport, HttpServletResponse response) throws Exception {
		if (isExport == null || !isExport) {
			List<DyLaikeShopEvaltion> list = douyinShopEvaltionService.getList(fraName, shopName, poiId, isExport, page,
					limit);
			return ResponseUtil.ok(new PageInfo<>(list));
		} else {
			List<DyLaikeShopEvaltion> list = douyinShopEvaltionService.getList(fraName, shopName, poiId, isExport, page,
					limit);
			ExcelUtils.exportExcel(list, null, "sheet1", DyLaikeShopEvaltion.class, "douyinpage.xls", response);
			return null;
		}
	}

	@ApiOperation("查询抖音门店违规三率数据")
	@GetMapping("getStoreThreeRate")
	public Object getStoreThreeRate(@RequestParam(value = "fraName", required = false) String fraName,
			@RequestParam(value = "shopName", required = false) String shopName,
			@RequestParam(value = "poiId", required = false) String poiId,
			@RequestParam(value = "goals", required = false) String goals, @RequestParam("page") Integer page,
			@RequestParam("limit") Integer limit, Boolean isExport, HttpServletResponse response) throws Exception {
		if (isExport == null || !isExport) {
			List<DyLaikeShopThreeRate> list = douyinShopThreeRateService.getList(fraName, shopName, poiId, isExport,
					page, limit, goals);
			return ResponseUtil.ok(new PageInfo<>(list));
		} else {
			List<DyLaikeShopThreeRate> list = douyinShopThreeRateService.getList(fraName, shopName, poiId, isExport,
					page, limit, goals);
			ExcelUtils.exportExcel(list, null, "sheet1", DyLaikeShopThreeRate.class, "douyinpage.xls", response);
			return null;
		}
	}

	@ApiOperation("查询抖音优惠券汇总")
	@GetMapping("getDYCouponSummary")
	public Object getDYCouponSummary(String couponName, @RequestParam("page") Integer page,
			@RequestParam("limit") Integer limit, Boolean isExport, HttpServletResponse response) throws Exception {
		if (isExport == null || isExport == false) {
			PageHelper.startPage(page, limit);
			List<DouyinDouponTotalVo> dyCouponSummary = dyCouponMapper.getDYCouponSummary(couponName);
			return ResponseUtil.ok(new PageInfo<>(dyCouponSummary));
		} else {
			List<DouyinDouponTotalVo> list = dyCouponMapper.getDYCouponSummary(couponName);
			ExcelUtils.exportExcel(list, null, "Sheet1", DouyinDouponTotalVo.class, "抖音优惠券汇总", response);
			return null;
		}
	}

	@ApiOperation("查询抖音优惠券明细")
	@GetMapping("getDYCouponDetail")
	public Object getDYCouponDetail(String couponName, @RequestParam("page") Integer page,
			@RequestParam("limit") Integer limit, Boolean isExport, HttpServletResponse response) throws Exception {
		if (isExport == null || isExport == false) {
			PageHelper.startPage(page, limit);
			List<DouyinCouponDetailVo> dyCouponSummary = dyCouponMapper.getDYCouponDetail(couponName);
			return ResponseUtil.ok(new PageInfo<>(dyCouponSummary));
		} else {
			List<DouyinCouponDetailVo> list = dyCouponMapper.getDYCouponDetail(couponName);
			ExcelUtils.exportExcel(list, null, "Sheet1", DouyinCouponDetailVo.class, "抖音优惠券明细", response);
			return null;
		}
	}

	@ApiOperation("查询小程序页面日志")
	@GetMapping("getAppPageLog")
	public Object getAppPageLog(@RequestParam("startTime") String startTime, @RequestParam("endTime") String endTime,
			@RequestParam("page") Integer page, @RequestParam("limit") Integer limit, Boolean isExport,
			Integer goodsId,Integer pageId,Integer eventId,
			HttpServletResponse response) throws Exception {

		if (StringUtils.isBlank(startTime) || StringUtils.isBlank(endTime)) {
			return ResponseUtil.fail("请选择开始和结束时间");
		}
		startTime = startTime + " 00:00:00";
		endTime = endTime + " 23:59:59";

		if (isExport == null || isExport == false) {
			PageHelper.startPage(page, limit);
			List<AppPageLogData> list = douyinPageService.getPageLog(startTime, endTime, goodsId,pageId,eventId);
			return ResponseUtil.ok(new PageInfo<>(list));
		} else {

			long dayByMinusDate = DateUtil.getDayByMinusDate(DateUtil.getDate(startTime), DateUtil.getDate(endTime));
			if (dayByMinusDate > 8L) {
				return ResponseUtil.fail("导出数据不能超过7天");
			}
			List<AppPageLogData> list = douyinPageService.getPageLog(startTime, endTime, goodsId,pageId,eventId);
			ExcelUtils.exportExcel(list, null, "Sheet1", AppPageLogData.class, "douyinpagelog.xls", response);
			return null;
		}
	}

	@ApiOperation("抖音商品")
	@GetMapping("getDyGoods")
	public Object getDyGoods() {
		return ResponseUtil.ok(douyinPageService.getDyGoods());
	}

	@ApiOperation("抖音商品")
	@GetMapping("getPageLogDict")
	public Object getPageLogDict() {
		Map<String, Object> data=new HashMap<String, Object>();
		data.put("pageDict", douyinPageDataMapper.selectPage());
		data.put("eventDict", douyinPageDataMapper.selectPageEvent());
		return ResponseUtil.ok(data);
	}

}
