package com.jzlife.report.controller;

import com.google.common.collect.Lists;
import com.jzlife.report.anno.LogRecordFiled;
import com.jzlife.report.dao.douyin.DouyinBoardMapper;
import com.jzlife.report.dao.douyin.DouyinRebuyMapper;
import com.jzlife.report.dto.EchartsLine;
import com.jzlife.report.dto.ReportNewCustomerTradeDetail;
import com.jzlife.report.dto.ReportOldCustomerRebuyDetail;
import com.jzlife.report.dto.douyin.DouyinBoardLineVo;
import com.jzlife.report.dto.douyin.DouyinBoardMapVo;
import com.jzlife.report.dto.douyin.DouyinRebuySingleDetailVo;
import com.jzlife.report.util.DateUtil;
import com.jzlife.report.util.DateUtils;
import com.jzlife.report.util.ExcelUtils;
import com.jzlife.report.util.ResponseUtil;
import com.jzlife.report.util.page.BeanUtil;
import com.jzlife.report.util.page.PagedResult;
import com.jzlife.report.vo.DayReportBaseVo;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.time.LocalDateTime;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 抖音私域复购相关报表
 */
@RestController
@RequestMapping("douyinRebuy")
public class DouyinRebuyController {

    @Resource
    private DouyinRebuyMapper rebuyMapper;

    @ApiOperation(value = "抖音私域复购相关报表", httpMethod = "GET", notes = "散单服务复购明细")
    @GetMapping("single")
    @LogRecordFiled(funName = "散单服务复购明细")
    public Object getSingle(
            @RequestParam(required = true) String city,
            @RequestParam(required = true) String startTime,
            @RequestParam(required = true) String endTime,
            @RequestParam(required = false) String workerName,
            @RequestParam(required = false) String rebuyStatus,
            @RequestParam(defaultValue = "1") Integer pageIndex,
            @RequestParam(defaultValue = "50") Integer pageSize,
            @RequestParam(defaultValue = "false") Boolean isExport,
            HttpServletResponse response) throws Exception {
        if (StringUtils.isBlank(startTime)) {
            return ResponseUtil.fail("服务日期不能为空");
        }

        if (isExport) {
            pageIndex = 1;
            pageSize = 100000;
        }

        List<DouyinRebuySingleDetailVo> list = rebuyMapper.getDouyinRebuySingleDetai(startTime, endTime, city, workerName, rebuyStatus, pageIndex, pageSize);
        if (isExport == false) {
            Integer total = list.size() > 0 ? list.get(0).getRowTotal() : 0;
            PagedResult<DouyinRebuySingleDetailVo> pagedResult = BeanUtil.toPagedResult(list);
            pagedResult.setTotal(total);
            pagedResult.setPageNo(pageIndex);
            pagedResult.setPageSize(pageSize);
            pagedResult.setStartPage(1);
            pagedResult.setEndPage((total / pageSize) + 1);
            pagedResult.setPages(pagedResult.getEndPage());
            return ResponseUtil.ok(pagedResult);
        } else {
            ExcelUtils.exportExcel(list, null, "Sheet1", DouyinRebuySingleDetailVo.class, "散单新客首单服务明细.xls", response);
            return null;
        }
    }

    @ApiOperation(value = "抖音私域复购相关报表", httpMethod = "GET", notes = "周期老客续约率")
    @GetMapping("cycle")
    @LogRecordFiled(funName = "周期老客续约率")
    public Object getCycle(
            @RequestParam(required = true) String city,
            @RequestParam(required = true) String startTime,
            @RequestParam(required = true) String endTime,
            @RequestParam(required = false) String workerName,
            @RequestParam(required = false) String rebuyStatus,
            @RequestParam(defaultValue = "1") Integer pageIndex,
            @RequestParam(defaultValue = "50") Integer pageSize,
            @RequestParam(defaultValue = "false") Boolean isExport,
            HttpServletResponse response) throws Exception {
        if (StringUtils.isBlank(startTime)) {
            return ResponseUtil.fail("服务日期不能为空");
        }

        if (isExport) {
            pageIndex = 1;
            pageSize = 100000;
        }

        List<ReportOldCustomerRebuyDetail> list = rebuyMapper.getDouyinRebuyCycleDetai(startTime, endTime, city, workerName, rebuyStatus, pageIndex, pageSize);
        if (isExport == false) {
            Integer total = list.size() > 0 ? list.get(0).getRowTotal() : 0;
            PagedResult<ReportOldCustomerRebuyDetail> pagedResult = BeanUtil.toPagedResult(list);
            pagedResult.setTotal(total);
            pagedResult.setPageNo(pageIndex);
            pagedResult.setPageSize(pageSize);
            pagedResult.setStartPage(1);
            pagedResult.setEndPage((total / pageSize) + 1);
            pagedResult.setPages(pagedResult.getEndPage());
            return ResponseUtil.ok(pagedResult);
        } else {
            ExcelUtils.exportExcel(list, null, "Sheet1", ReportOldCustomerRebuyDetail.class, "散单新客首单服务明细.xls", response);
            return null;
        }
    }
}
