package com.jzlife.report.controller;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;
import org.apache.ibatis.annotations.Param;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.google.common.collect.Lists;
import com.jzlife.report.anno.LogRecordFiled;
import com.jzlife.report.dao.reports.ReportPrivateDomainMapper;
import com.jzlife.report.dto.douyin.DouyinBoardMapVo;
import com.jzlife.report.service.report.ReportPrivateDomainService;
import com.jzlife.report.util.DateUtil;
import com.jzlife.report.util.ResponseUtil;

/**
 * 私域看板
 * 
 * @author 阿闰
 *
 */
@RestController
@RequestMapping("privateDomain")
public class PrivateDomainController {

	@Autowired
	ReportPrivateDomainMapper reportPrivateDomainMapper;
	
	@Autowired
	ReportPrivateDomainService reportPrivateDomainService;

	@GetMapping("topSalesNumber")
	@LogRecordFiled(funName = "顶部的销售统计和加盟商数量")
	public Object topSalesNumber() {
		Map<String, Object> daySales = reportPrivateDomainMapper.daySales(DateUtil.getBeforeDay(-1));
		Map<String, Object> monthSales = reportPrivateDomainMapper.monthSales(DateUtil.getBeforeMonth(-1));
		Map<String, Object> jmsNumber = reportPrivateDomainMapper.jmsNumber(DateUtil.getBeforeMonth(-1));

		Map<String, Object> data = new HashMap<String, Object>();
		data.put("daySales", daySales);
		data.put("monthSales", monthSales);
		data.put("jmsNumber", jmsNumber);
		return ResponseUtil.ok(data);
	}
	
	@GetMapping("getAllRank")
	@LogRecordFiled(funName = "各销售排行,type 固定查某类，dayOrMonth查某天或某月 1天，2月")
	public Object getAllRank(Integer type,Integer dayOrMonth) {
		if(dayOrMonth==null) {
			return ResponseUtil.fail("dayOrMonth不能为空");
		}
		return reportPrivateDomainService.getAllRank(type, dayOrMonth);
	}

	@GetMapping("getServicedHotMap")
	@LogRecordFiled(funName = "获取核销热力图")
	public Object getServicedHotMap(Integer days) {
		List<DouyinBoardMapVo> list = reportPrivateDomainMapper.getServicedHotMap(days);
		List<String> result = Lists.newArrayList();
		list.forEach(v -> {
			if (StringUtils.isNotEmpty(v.getPosition()) && v.getPosition().equals(",") == false)
				result.add(v.getPosition());
			else {
				result.add(getCityLnglat(v.getCity()));
			}
		});
		return ResponseUtil.ok(result);
	}

	String getCityLnglat(String city) {
		city = city == null ? "上海" : city.replace("市", "");
		switch (city) {
		case "北京":
			return "116.720996, 39.903666";
		case "深圳":
			return "114.05522, 22.542207";
		case "南京":
			return "118.665131, 31.930355";
		case "苏州":
			return "120.58053, 31.302569";
		case "杭州":
			return "120.20805, 30.247872";
		case "重庆":
			return "106.511589,29.532241";
		case "成都":
			return "104.065521,30.657529";
		case "青岛":
			return "120.371836,36.084444";
		case "昆山":
			return "121.074918,31.306858";
		case "广州":
			return "113.253725,23.12607";
		default:// 默认上海
			return "121.47102, 31.229203";
		}
	}

}
