package com.jzlife.report.controller;

import com.jzlife.report.anno.LogRecordFiled;
import com.jzlife.report.dto.ServiceGmvDto;
import com.jzlife.report.entity.hmorder.DiMerchandiseDisplay;
import com.jzlife.report.service.hmorder.MerchandiseDisplayService;
import com.jzlife.report.service.hmorder.ReportService;
import com.jzlife.report.service.hmorder.StationSerevice;
import com.jzlife.report.util.ExcelUtil;
import com.jzlife.report.util.ResponseUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * @author fuxubing
 */
@Api(value = "Report-Controller", description = "报表")
@RestController
@RequestMapping(value = "report")
public class ReportController {
	@Resource
	private ReportService reportService;

	@Autowired
	StationSerevice stationSerevice;

	@Autowired
	MerchandiseDisplayService merchandiseDisplayService;

	@ApiOperation(value = "查询日销售统计报表")
	@GetMapping("getSaleReportData")
	@LogRecordFiled(funName = "日销售统计")
	@ApiImplicitParams({ @ApiImplicitParam(name = "startTime", value = "开始时间", paramType = "query"),
			@ApiImplicitParam(name = "endTime", value = "结束时间", paramType = "query"),
			@ApiImplicitParam(name = "city", value = "城市", paramType = "query"),
			@ApiImplicitParam(name = "storeNames", value = "门店（多个，分割）", paramType = "query"),
			@ApiImplicitParam(name = "businessLine", value = "业务线", paramType = "query"),
			@ApiImplicitParam(name = "levelOnes", value = "一级分类（多个，分割）", paramType = "query"),
			@ApiImplicitParam(name = "levelTwos", value = "二级分类（多个，分割）", paramType = "query") })
	public Object getReportData(String startTime, String endTime, String city, String storeNames, String businessLine,
			String levelOnes, String levelTwos) {
		Map<String, Object> saleReportData = reportService.getSaleReportData(startTime, endTime, city, storeNames,
				businessLine, levelOnes, levelTwos);
		return ResponseUtil.ok(saleReportData);
	}

	@ApiOperation(value = "查询月销售统计报表")
	@GetMapping("getMonthlySaleReportData")
	@ApiImplicitParams({ @ApiImplicitParam(name = "beginMonth", value = "开始月份", paramType = "query"),
			@ApiImplicitParam(name = "endMonth", value = "结束月份", paramType = "query"),
			@ApiImplicitParam(name = "city", value = "城市", paramType = "query"),
			@ApiImplicitParam(name = "storeNames", value = "门店（多个，分割）", paramType = "query"),
			@ApiImplicitParam(name = "businessLine", value = "业务线", paramType = "query"),
			@ApiImplicitParam(name = "levelOnes", value = "一级分类（多个，分割）", paramType = "query"),
			@ApiImplicitParam(name = "levelTwos", value = "二级分类（多个，分割）", paramType = "query") })
	@LogRecordFiled(funName = "月销售统计")
	public Object getMonthlySaleReportData(String beginMonth, String endMonth, String city, String storeNames,
			String businessLine, String levelOnes, String levelTwos) {
		Map<String, Object> saleReportData = reportService.getMonthlySaleReportData(beginMonth, endMonth, city,
				storeNames, businessLine, levelOnes, levelTwos);
		return ResponseUtil.ok(saleReportData);
	}

	@ApiOperation(value = "服务gmv统计报表表格")
	@GetMapping("getServiceGmv")
	@LogRecordFiled(funName = "服务gmv统计报表")
	@ApiImplicitParams({ @ApiImplicitParam(name = "startTime", value = "开始日期", paramType = "query"),
			@ApiImplicitParam(name = "endTime", value = "结束日期", paramType = "query") })
	public Object getServiceGmv(String startTime, String endTime) {
		if (StringUtils.isBlank(startTime) || StringUtils.isBlank(endTime)) {
			return ResponseUtil.fail("日期不能为空");
		}

		Map<Object, Object> gmvData = reportService.getGmvData(startTime, endTime);
		return ResponseUtil.ok(gmvData);
	}

	@ApiOperation(value = "服务gmv统计报表导出")
	@GetMapping("exportServiceGmvExport")
	@LogRecordFiled(funName = "服务gmv统计报表导出")
	@ApiImplicitParams({ @ApiImplicitParam(name = "startTime", value = "开始日期", paramType = "query"),
			@ApiImplicitParam(name = "endTime", value = "结束日期", paramType = "query") })
	public Object exportServiceGmvExport(String startTime, String endTime, HttpServletResponse response) {
		if (StringUtils.isBlank(startTime) || StringUtils.isBlank(endTime)) {
			return ResponseUtil.fail("日期不能为空");
		}

		Map<Object, Object> gmvData = reportService.getGmvData(startTime, endTime);

		List<DiMerchandiseDisplay> diList = (List<DiMerchandiseDisplay>) gmvData.get("diList");
		List<ServiceGmvDto> listGmv = (List<ServiceGmvDto>) gmvData.get("listGmv");

		ExcelUtil eu = new ExcelUtil();
		String title = "服务gmv统计报表";
		String[] rowsName = new String[diList.size() + 2];
		List<Object[]> dataList = new ArrayList<Object[]>();

		int i = 0;
		rowsName[i++] = "城市";
		rowsName[i++] = "单量或金额";
		for (DiMerchandiseDisplay d : diList) {
			rowsName[i++] = d.getAttr1();
		}

		for (ServiceGmvDto g : listGmv) {
			Object[] oj = new Object[diList.size() + 2];
			int c = 0;
			oj[c++] = g.getCity();
			oj[c++] = g.getName();

			for (String s : rowsName) {
				if("城市".equals(s)||"单量或金额".equals(s)) {
					continue;
				}
				oj[c++] = g.getSumOramount().get(s);
			}
			dataList.add(oj);
		}

		try {
			String fileName = new String("服务gmv统计报表.xlsx".getBytes("UTF-8"), "iso-8859-1");
			eu.exportExcel(title, rowsName, dataList, fileName, response);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		return null;
	}

	@GetMapping("getCityStation")
	public Object getCityStation() {
		Map<Object, Object> dataMap = stationSerevice.getList();
		return ResponseUtil.ok(dataMap);
	}

	@GetMapping("getdisplay")
	public Object getdisplay() {
		List<DiMerchandiseDisplay> list = merchandiseDisplayService.getList();
		return ResponseUtil.ok(list);
	}

}
