package com.jzlife.report.controller;

import cn.afterturn.easypoi.excel.entity.ExportParams;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.jzlife.report.anno.LogRecordFiled;
import com.jzlife.report.dao.hmorder.SalesOrderMapper;
import com.jzlife.report.dto.BuzLineKewordDto;
import com.jzlife.report.dto.HmSalesDto;
import com.jzlife.report.dto.SaleAppDto;
import com.jzlife.report.dto.export.*;
import com.jzlife.report.excelExport.DayBDInvertDetail;
import com.jzlife.report.excelExport.numberConvert.ExcelStyleUtil;
import com.jzlife.report.schedule.TaskReportData;
import com.jzlife.report.service.hmorder.SalesOrderExportService;
import com.jzlife.report.service.newreport.PrivilegeManService;
import com.jzlife.report.service.newreport.SalesReportService;
import com.jzlife.report.service.nurse.CityStationService;
import com.jzlife.report.util.ExcelUtils;
import com.jzlife.report.util.ResponseUtil;
import com.jzlife.report.util.SortCityUtil;
import com.jzlife.report.util.UserUtil;
import com.jzlife.report.vo.DayReportBaseVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.compress.utils.Lists;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

@Api(value = "销售数据报表", description = "销售数据报表")
@RequestMapping("salesdata")
@RestController
public class SalesDataReportController {

    @Autowired
    SalesReportService salesReportService;

    @Autowired
    SalesOrderMapper salesOrderMapper;

    @Autowired
    SalesOrderExportService salesOrderExportService;

    @Autowired
    TaskReportData task;

    @Autowired
    CityStationService cityStationService;

    @Autowired
    UserUtil userUtil;

    @Autowired
    PrivilegeManService privilegeManService;

    @ApiOperation("刷新销售缓存数据")
    @GetMapping("refCacheSale")
    public Object refCacheSale(String startMonth, String endMonth) {
        new Thread() {
            public void run() {
                task.insertSalesCache(startMonth, endMonth);
            }

            ;
        }.start();
        return ResponseUtil.ok();
    }

    @ApiOperation("公司销售GMV目标")
    @GetMapping("getCSGmvTarget")
    @LogRecordFiled(funName = "公司销售GMV目标")
    public Object getCompanySalesGmvTarget(HttpServletRequest request) {
        Integer userId = userUtil.getUser(request);
        if (userId == null) {
            return ResponseUtil.fail402();
        }
        List<String> userCity = cityStationService.getUserCity(userId);
        if (null == userCity || userCity.size() == 0) {
            return ResponseUtil.fail402();
        }
        String city = String.join(",", userCity);

        boolean boo = privilegeManService.getPrivilegeManByUserId(userId);
        if (boo) {
            city = null;
        }

        Map<String, Object> data = salesReportService.getCompanySalesGmvTarget(city);
        return ResponseUtil.ok(data);
    }

    @ApiOperation("公司销售GMV目标和复购情况 2021-06")
    @GetMapping("getCSGmvData")
    @LogRecordFiled(funName = "公司销售GMV目标和复购情况")
    public Object getCompanySalesGmvData(String yearMonth, HttpServletRequest request) {
        Integer userId = userUtil.getUser(request);
        if (userId == null) {
            return ResponseUtil.fail402();
        }
        List<String> userCity = cityStationService.getUserCity(userId);
        if (null == userCity || userCity.size() == 0) {
            return ResponseUtil.fail402();
        }

        if (StringUtils.isBlank(yearMonth)) {
            return ResponseUtil.fail("请选择月份");
        }
        String city = String.join(",", userCity);
        String sortCity = SortCityUtil.sortCity(city);
        boolean boo = privilegeManService.getPrivilegeManByUserId(userId);
        if (boo) {// 全部城市权限，不要带城市，查所有
            sortCity = null;
        }

        Map<String, Object> companySalesGmvData = salesReportService.getCompanySalesGmvData(yearMonth, sortCity);

        return ResponseUtil.ok(companySalesGmvData);
    }

    @ApiOperation("实销额与单量  timeType 1天2月")
    @GetMapping("getCSTrendchart")
    @LogRecordFiled(funName = "实销额与单量")
    public Object getCSTrendchart(String channel, String city, String businessTypeName, String businessLine,
                                  Integer timeType, String startTime, String endTime, String startMonth, String endMonth,
                                  HttpServletRequest request) {

        if (StringUtils.isBlank(businessTypeName)) {
            // businessTypeName = "保洁,保姆,品类,搬家,附加,普通保洁单次,普通保洁周期,鲸选保洁单次,鲸选保洁周期";
            businessTypeName = "保洁,保姆,品类,搬家,附加";
            // 普通保洁单次,普通保洁周期,鲸选保洁单次,鲸选保洁周期 作为 businessLine
        }

        if (StringUtils.isBlank(city)) {
            // return ResponseUtil.fail("请选择城市");
            city = cityStationService.getUserCityString(userUtil.getUser(request));
            //如果多于70个城市 表示全量
            if (city.split(",").length > 70)
                city = null;
        }
        String sh = ",上海浦东,上海浦西,上海公共,上海";
        if (city != null && city.contains("上海")) {
            city = city + sh;
        }

        if (timeType == 1 && (StringUtils.isBlank(startTime) || StringUtils.isBlank(endTime))) {
            return ResponseUtil.fail("请选择日期");
        }
        if (timeType == 2 && (StringUtils.isBlank(startMonth) || StringUtils.isBlank(endMonth))) {
            return ResponseUtil.fail("请选择月份");
        }

        Map<String, Object> data = salesReportService.getCompanySalesbrokenLine(channel, city, businessTypeName,
                businessLine, timeType, startTime, endTime, startMonth, endMonth);

        return ResponseUtil.ok(data);
    }

    @ApiOperation("获取保洁商品属性下拉框")
    @GetMapping("getBjGoodsAttr")
    public Object getBjGoodsAttr() {
        return ResponseUtil.ok(salesReportService.getBjGoodsArrt());
    }

    @ApiOperation("服务GMV与单量 timeType 1天2月")
    @GetMapping("getOrderNumberchart")
    @LogRecordFiled(funName = "服务GMV与单量")
    public Object getOrderNumberchart(String city, String businessTypeName, Integer timeType, String startTime,
                                      String endTime, String startMonth, String endMonth, HttpServletRequest request) {

        if (StringUtils.isBlank(businessTypeName)) {
            // businessTypeName = "保洁,保姆,品类,搬家,附加,普通保洁单次,普通保洁周期,鲸选保洁单次,鲸选保洁周期";
            businessTypeName = "保洁,品类";
            // 普通保洁单次,普通保洁周期,鲸选保洁单次,鲸选保洁周期 作为 businessLine
        }

        if (StringUtils.isBlank(city)) {
            // return ResponseUtil.fail("请选择城市");
            city = cityStationService.getUserCityString(userUtil.getUser(request));
        }

        String sh = ",上海浦东,上海浦西,上海公共,上海";
        if (city != null && city.contains("上海")) {
            city = city + sh;
        }
        if (timeType == 1 && (StringUtils.isBlank(startTime) || StringUtils.isBlank(endTime))) {
            return ResponseUtil.fail("请选择日期");
        }
        if (timeType == 2 && (StringUtils.isBlank(startMonth) || StringUtils.isBlank(endMonth))) {
            return ResponseUtil.fail("请选择月份");
        }

        Map<String, Object> data = salesReportService.getOrderNumberchart(city, businessTypeName, timeType, startTime,
                endTime, startMonth, endMonth);

        return ResponseUtil.ok(data);
    }

    @ApiOperation("获取来源渠道--订单")
    @GetMapping("getChannel")
    public Object getChannel() {
        List<String> list = salesOrderMapper.getChannel();
        return ResponseUtil.ok(list);
    }

    @ApiOperation("获取保姆来源")
    @GetMapping("getNurseSource")
    public Object getNurseSource() {
        List<String> list = salesOrderMapper.getNurseSource();
        return ResponseUtil.ok(list);
    }

    @ApiOperation("小程序订单列表")
    @GetMapping("selectLittleAppOrder")
    @LogRecordFiled(funName = "小程序订单列表")
    public Object selectLittleAppOrder(@RequestParam(defaultValue = "1") Integer pageNo,
                                       @RequestParam(defaultValue = "50") Integer pageSize, String startDate, String endDate, String channel,
                                       String city, String levelOne, @RequestParam(defaultValue = "false") Boolean isExport,
                                       HttpServletResponse response, HttpServletRequest request) throws Exception {
        if (StringUtils.isBlank(startDate) || StringUtils.isBlank(endDate)) {
            return ResponseUtil.fail("时间范围不能为空");
        }
        if (StringUtils.isBlank(city)) {
            // return ResponseUtil.fail("请选择城市");
            city = cityStationService.getUserCityString(userUtil.getUser(request));
        }
        if (city.contains("全国")) {
            city = null;
        }

        startDate = startDate + " 00:00:00";
        endDate = endDate + " 23:59:59";
        Object result = salesOrderExportService.selectLittleAppOrder(pageNo, pageSize, startDate, endDate, channel,
                city, levelOne, isExport);
        if (isExport == false)
            return result;
        else {
            ExcelUtils.exportExcel((List<SaleAppDto>) result, null, "Sheet1", SaleAppDto.class, "小程序订单列表.xls",
                    response);
            return null;
        }
    }


    @ApiOperation("各渠道销售明细列表")
    @GetMapping("selectChannelSales")
    @LogRecordFiled(funName = "各渠道销售明细列表")
    public Object selectChannelSales(@RequestParam(defaultValue = "1") Integer pageNo,
                                     @RequestParam(defaultValue = "50") Integer pageSize, String startDate, String endDate, String channel,
                                     String city, String levelOne, @RequestParam(defaultValue = "false") Boolean isExport,
                                     HttpServletRequest request, HttpServletResponse response) throws Exception {
        startDate = startDate + " 00:00:00";
        endDate = endDate + " 23:59:59";
        if (StringUtils.isBlank(city)) {
            // return ResponseUtil.fail("请选择城市");

            city = cityStationService.getUserCityString(userUtil.getUser(request));
        }
        Object result = salesOrderExportService.selectChannelSales(pageNo, pageSize, startDate, endDate, channel, city,
                levelOne, isExport);
        if (isExport) {// 导出
            List<ChannelSalesDto> list = (List<ChannelSalesDto>) result;
            if (null != list && list.size() > 0) {
                ExcelUtils.exportExcel(list, null, "Sheet1", ChannelSalesDto.class, "各渠道销售明细.xls", response);
            }
            return null;
        }
        return result;

    }

    @ApiOperation("销售汇总列表")
    @GetMapping("salesSummary")
    @LogRecordFiled(funName = "销售汇总列表")
    public Object salesSummary(@RequestParam(defaultValue = "1") Integer pageNo,
                               @RequestParam(defaultValue = "50") Integer pageSize, String startDate, String endDate, String channel,
                               String city, String levelOne, HttpServletRequest request) {
        startDate = startDate + " 00:00:00";
        endDate = endDate + " 23:59:59";
        if (StringUtils.isBlank(city)) {
            // return ResponseUtil.fail("请选择城市");
            city = cityStationService.getUserCityString(userUtil.getUser(request));
        }

        if (city.contains("全国")) {
            city = null;
        }
        return salesOrderExportService.salesSummary(pageNo, pageSize, startDate, endDate, channel, city, levelOne);
    }

    @ApiOperation("销售汇总导出")
    @GetMapping("salesSummaryExport")
    @LogRecordFiled(funName = "销售汇总导出")
    public void salesSummaryExport(String startDate, String endDate, String channel, String city, String levelOne,
                                   HttpServletResponse response, HttpServletRequest request) throws Exception {
        startDate = startDate + " 00:00:00";
        endDate = endDate + " 23:59:59";
        if (StringUtils.isBlank(city)) {
            // return;
            city = cityStationService.getUserCityString(userUtil.getUser(request));
        }
        if (city.contains("全国")) {
            city = null;
        }
        List<SalesReportExport> list = salesOrderExportService.salesSummaryExport(startDate, endDate, channel, city,
                levelOne);

        ExcelUtils.exportExcel(list, null, "Sheet1", SalesReportExport.class, "销售汇总.xls", response);
    }

    @ApiOperation("服务GMV汇总")
    @GetMapping("serviceGMVCount")
    @LogRecordFiled(funName = "服务GMV汇总")
    public Object serviceGMVCount(@RequestParam(defaultValue = "1") Integer pageNo,
                                  @RequestParam(defaultValue = "50") Integer pageSize,
                                  String startDate, String endDate, String city,
                                  String levelOne,
                                  String status,
                                  String channel,
                                  @RequestParam(defaultValue = "false") Boolean isExport, HttpServletRequest request,
                                  HttpServletResponse response) throws Exception {
        startDate = startDate + " 00:00:00";
        endDate = endDate + " 23:59:59";
        if (StringUtils.isBlank(city)) {
            city = cityStationService.getUserCityString(userUtil.getUser(request));
        }
        if (city.contains("全国")) {
            city = null;
        }

        Object result = salesOrderExportService.serviceGMVCount(pageNo, pageSize, startDate, endDate, city, levelOne, isExport, status, channel);
        if (isExport) {// 导出
            List<ServiceGMVCountExportDto> list = (List<ServiceGMVCountExportDto>) result;
            if (list != null && list.size() > 0) {
                ExcelUtils.exportExcel(list, null, "Sheet1", ServiceGMVCountExportDto.class, "服务GMV汇总.xls", response);
            }
            return null;
        }
        return result;

    }

    @ApiOperation("服务GMV汇总导出")
    @GetMapping("serviceGMVCountExport")
    @LogRecordFiled(funName = "服务GMV汇总导出")
    public void serviceGMVCountExport(String startDate, String endDate, String city, String levelOne, String status, String channel,
                                      HttpServletResponse response, HttpServletRequest request) throws Exception {
        startDate = startDate + " 00:00:00";
        endDate = endDate + " 23:59:59";
        if (StringUtils.isBlank(city)) {
            city = cityStationService.getUserCityString(userUtil.getUser(request));
        }
        if (city.contains("全国")) {
            city = null;
        }

        List<ServiceGmvExport> list = salesOrderExportService.serviceGMVCountExport(startDate, endDate, city, levelOne, status, channel);
        ExcelUtils.exportExcel(list, null, "Sheet1", ServiceGmvExport.class, "服务GMV汇总.xls", response);
    }

    @ApiOperation("月销售进度回溯（每日）")
    @GetMapping("monthSalesPlan")
    @LogRecordFiled(funName = "月销售进度回溯（每日）")
    public Object monthSalesPlan(@RequestParam(defaultValue = "1") Integer pageNo,
                                 @RequestParam(defaultValue = "50") Integer pageSize, String startDate, String city, String levelOne,
                                 @RequestParam(defaultValue = "false") Boolean isExport, HttpServletRequest request,
                                 HttpServletResponse response) throws Exception {
        startDate = startDate + " 00:00:00";
        String endDate = startDate + " 23:59:59";// String endDate,
        if (StringUtils.isBlank(city)) {
            // return ResponseUtil.fail("请选择城市");

            city = cityStationService.getUserCityString(userUtil.getUser(request));
        }

        Object result = salesOrderExportService.monthSalesPlan(pageNo, pageSize, startDate, endDate, city, levelOne,
                isExport);
        if (isExport) {
            List<MonthSalesPlanExportDto> list = (List<MonthSalesPlanExportDto>) result;
            if (list != null && list.size() > 0) {
                ExcelUtils.exportExcel(list, null, "Sheet1", MonthSalesPlanExportDto.class, "月销售进度回溯xls", response);
            }
            return null;
        }

        return result;

    }

    @ApiOperation("非盒马渠道当月实销额扇形图")
    @GetMapping("notHMActualSales")
    @LogRecordFiled(funName = "非盒马渠道当月实销额扇形图")
    public Object notHMActualSales(String startDate, String endDate) {
        if (StringUtils.isBlank(startDate) || StringUtils.isBlank(endDate)) {
            return ResponseUtil.fail("时间不能为空");
        }
        startDate = startDate + " 00:00:00";
        endDate = endDate + " 23:59:59";
        return salesOrderExportService.notHMActualSales(startDate, endDate);
    }

    @ApiOperation("线索列表")
    @GetMapping("selectClue")
    @LogRecordFiled(funName = "线索列表")
    public Object selectClue(@RequestParam(defaultValue = "1") Integer pageNo,
                             @RequestParam(defaultValue = "50") Integer pageSize, String startDate, String endDate, String resource,
                             String city, String serviceModel, @RequestParam(defaultValue = "false") Boolean isExport,
                             HttpServletRequest request, HttpServletResponse response) throws Exception {
        startDate = startDate + " 00:00:00";
        endDate = endDate + " 23:59:59";

        if (StringUtils.isBlank(city)) {
            // return ResponseUtil.fail("请选择城市");
            city = cityStationService.getUserCityString(userUtil.getUser(request));
        }
        Object result = salesOrderExportService.selectClue(pageNo, pageSize, startDate, endDate, resource, city,
                serviceModel, isExport);
        if (isExport) {
            List<SelectClueDtoExport> list = (List<SelectClueDtoExport>) result;
            if (list != null && list.size() > 0) {
                ExcelUtils.exportExcel(list, null, "Sheet1", SelectClueDtoExport.class, "线索列表.xls", response);
            }
        }

        return salesOrderExportService.selectClue(pageNo, pageSize, startDate, endDate, resource, city, serviceModel,
                isExport);
    }

    @ApiOperation("保姆签单")
    @GetMapping("servantSignOrder")
    @LogRecordFiled(funName = "保姆签单")
    public Object servantSignOrder(@RequestParam(defaultValue = "1") Integer pageNo,
                                   @RequestParam(defaultValue = "50") Integer pageSize, String startDate, String endDate, String city,
                                   String remark, String serviceModel, @RequestParam(defaultValue = "false") Boolean isExport,
                                   HttpServletRequest request, HttpServletResponse response) throws Exception {
        startDate = startDate + " 00:00:00";
        endDate = endDate + " 23:59:59";

        if (StringUtils.isBlank(city)) {
            // return ResponseUtil.fail("请选择城市");
            city = cityStationService.getUserCityString(userUtil.getUser(request));
        }

        Object result = salesOrderExportService.servantSignOrder(pageNo, pageSize, startDate, endDate, city, remark, serviceModel, isExport);
        if (isExport) {
            List<ServantSignOrderExport> list = (List<ServantSignOrderExport>) result;
            if (list != null && list.size() > 0) {
                ExcelUtils.exportExcel(list, null, "Sheet1", ServantSignOrderExport.class, "保姆签单.xls", response);
            }
        }
        return result;
    }

    @ApiOperation("支付记录")
    @GetMapping("cluePayOrder")
    @LogRecordFiled(funName = "支付记录")
    public Object cluePayOrder(@RequestParam(defaultValue = "1") Integer pageNo,
                               @RequestParam(defaultValue = "50") Integer pageSize, String startDate, String endDate,
                               @RequestParam(defaultValue = "false") Boolean isExport,
                               HttpServletRequest request, HttpServletResponse response) throws Exception {
        startDate = startDate + " 00:00:00";
        endDate = endDate + " 23:59:59";

        Object result = salesOrderExportService.cluePayOrder(pageNo, pageSize, startDate, endDate, isExport);
        if (isExport) {
            List<CluePayOrderExportDto> list = (List<CluePayOrderExportDto>) result;
            if (list != null && list.size() > 0) {
                ExcelUtils.exportExcel(list, null, "Sheet1", CluePayOrderExportDto.class, "支付记录.xls", response);
            }
        }
        return result;
    }


    @ApiOperation(value = "BD转化", httpMethod = "POST", notes = "时间格式为：yyyy-MM-dd,查询选定时间段内的BD转化")
    @PostMapping("bdInvertDetail")
    @LogRecordFiled(funName = "BD转化")
    public Object bdInvertDetail(@Validated @RequestBody DayReportBaseVo vo, HttpServletRequest request) {
        if (StringUtils.isEmpty(vo.getCity())) {
            String citys = request.getHeader("allowCity");
            if (StringUtils.isNotEmpty(citys)) {
                vo.setCity(citys);
            }
        }
        PageInfo<Map<String, Object>> data = salesReportService.bdInvertDetail(vo);
        return ResponseUtil.ok(data);
    }

    @ApiOperation(value = "BD转化导出", httpMethod = "POST", notes = "时间格式为：yyyy-MM-dd,查询选定时间段内的BD转化")
    @GetMapping("bdInvertDetailExport")
    @LogRecordFiled(funName = "BD转化导出")
    public void bdInvertDetailExport(
            @RequestParam(value = "city", required = false) String city,
            @RequestParam("startTime") String startTime,
            @RequestParam("endTime") String endTime,
            HttpServletRequest request, HttpServletResponse response) throws Exception {
        DayReportBaseVo vo = new DayReportBaseVo();
        if (StringUtils.isEmpty(city)) {
            String citys = request.getHeader("allowCity");
            if (StringUtils.isNotEmpty(citys)) {
                vo.setCity(citys);
            }
        }
        vo.setStartTime(startTime);
        vo.setEndTime(endTime);
        ExportParams exportParams = new ExportParams();
        exportParams.setStyle(ExcelStyleUtil.class);
        exportParams.setTitle("BD转化");
        exportParams.setSheetName("BD转化");
        String fileName = "BD转化-" + vo.getStartTime() + "至" + vo.getEndTime();
        List<Map<String, Object>> result = salesReportService.bdInvertDetailNoPage(vo);
        ExcelUtils.defaultExport(tran(result), DayBDInvertDetail.class, fileName, response, exportParams);
    }

    private List<DayBDInvertDetail> tran(List<Map<String, Object>> result) {
        List<DayBDInvertDetail> lists = new ArrayList<>();
        for (Map<String, Object> objectMap : result) {
            DayBDInvertDetail dayBDInvertDetail = new DayBDInvertDetail();
            dayBDInvertDetail.setCity((String) objectMap.get("city"));
            dayBDInvertDetail.setCreatName((String) objectMap.get("creat_name"));
            dayBDInvertDetail.setClueCount(((BigDecimal) objectMap.get("clue_count")).intValue());
            dayBDInvertDetail.setSignCount(((BigDecimal) objectMap.get("sign_count")).intValue());
            dayBDInvertDetail.setTotalPay(((BigDecimal) objectMap.get("total_pay")).doubleValue());
            dayBDInvertDetail.setInverstRate((String) objectMap.get("inverst_rate"));
            dayBDInvertDetail.setAvgPrice(((BigDecimal) objectMap.get("avg_price")).doubleValue());
            dayBDInvertDetail.setClueEffic(((BigDecimal) objectMap.get("clue_effic")).doubleValue());
            dayBDInvertDetail.setSignDay(((BigDecimal) objectMap.get("sign_day")).doubleValue());
            lists.add(dayBDInvertDetail);
        }
        return lists;
    }

    @ApiOperation("单品走势")
    @GetMapping("orderTrend")
    @LogRecordFiled(funName = "单品走势")
    public Object orderTrend(String startDate, String endDate, String city, String levelOne, String keyWord,
                             String channel, HttpServletRequest request) {
        if (StringUtils.isBlank(city)) {
            // return ResponseUtil.fail("请选择城市");
            city = cityStationService.getUserCityString(userUtil.getUser(request));
        }
        return salesOrderExportService.danpingzoushi(startDate, endDate, city, levelOne, keyWord, channel);
    }

    @ApiOperation("单品走势下拉框，业务线，关键字")
    @GetMapping("orderTrendSelect")
    public Object orderTrendSelect() {
        List<BuzLineKewordDto> buzLine = salesOrderExportService.getBuzLine();
        return ResponseUtil.ok(buzLine);
    }

    @ApiOperation("城市进度,某天,日期不能为空")
    @GetMapping("cityProgress")
    @LogRecordFiled(funName = "城市进度")
    public Object cityProgress(String date, @RequestParam(defaultValue = "false") Boolean isExport,
                               HttpServletRequest request, HttpServletResponse response) throws Exception {
        if (StringUtils.isBlank(date)) {
            return ResponseUtil.fail("日期不能为空");
        }
        List<CityProgressExportDto> cityProgress = salesOrderExportService.cityProgress(date, isExport);

        if (isExport) {
            if (cityProgress != null && cityProgress.size() > 0) {
                ExcelUtils.exportExcel(cityProgress, null, "Sheet1", CityProgressExportDto.class, "城市进度.xls", response);
            }
            return null;
        }
        return ResponseUtil.ok(cityProgress);

    }

    @ApiOperation("商城补差单")
    @GetMapping("diffOrderReport")
    @LogRecordFiled(funName = "商城补差单")
    public Object diffOrderReport(@RequestParam(defaultValue = "1") Integer pageNo,
                                  @RequestParam(defaultValue = "50") Integer pageSize, String startDate, String endDate,
                                  @RequestParam(defaultValue = "false") Boolean isExport,
                                  HttpServletRequest request, HttpServletResponse response) throws Exception {
        if (StringUtils.isBlank(startDate) || StringUtils.isBlank(endDate)) {
            return ResponseUtil.fail("请选择日期");
        }
        startDate = startDate + " 00:00:00";
        endDate = endDate + " 23:59:59";

        Object result = salesOrderExportService.selectDiffOrder(pageNo, pageSize, startDate, endDate, isExport);
        if (isExport) {
            List<MallDIffOrderExportDto> list = (List<MallDIffOrderExportDto>) result;
            if (list != null && list.size() > 0) {
                ExcelUtils.exportExcel(list, null, "Sheet1", MallDIffOrderExportDto.class, "商城补差单.xls", response);
            }
        }

        return salesOrderExportService.selectDiffOrder(pageNo, pageSize, startDate, endDate,
                isExport);
    }

    @ApiOperation("盒马销售明细")
    @GetMapping("hmSalesOrder")
    @LogRecordFiled(funName = "盒马销售明细")
    public Object hmSalesOrder(
            String startTime,
            String endTime,
            @RequestParam(defaultValue = "1") Integer pageNo,
                               @RequestParam(defaultValue = "50") Integer pageSize,
                               @RequestParam(defaultValue = "false") Boolean isExport,
                               String buyerPhone, HttpServletResponse response, HttpServletRequest request) throws Exception {
        if(isExport&&StringUtils.isEmpty(startTime)){
            return ResponseUtil.fail501("请选择支付日期");
        }

        List<HmSalesDto> list = Lists.newArrayList();
        if (isExport) {
            list = salesOrderExportService.selectHmSales(startTime, endTime, pageNo, pageSize, buyerPhone, isExport);
            if (list != null && list.size() > 0) {
                ExcelUtils.exportExcel(list, null, "Sheet1", HmSalesDto.class, "盒马销售明细.xls", response);
            }
            return null;
        } else {
            PageHelper.startPage(pageNo, pageSize);
            list = salesOrderExportService.selectHmSales(startTime, endTime, pageNo, pageSize, buyerPhone, isExport);

            return ResponseUtil.ok(new PageInfo<>(list));
        }
    }

}
