package com.jzlife.report.exception;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.WebDataBinder;
import org.springframework.web.bind.annotation.*;

import com.jzlife.report.util.ResponseUtil;

import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

import javax.servlet.http.HttpServletRequest;

@ControllerAdvice
public class ExceptionAdvice {


    private static final Logger logger = LoggerFactory.getLogger(ExceptionAdvice.class);

    /**
     * 应用到所有@RequestMapping注解方法，在其执行之前初始化数据绑定器
     *
     * @param binder
     */
    @InitBinder
    public void initBinder(WebDataBinder binder) {
    }

    /**
     * 把值绑定到Model中，使全局@RequestMapping可以获取到该值
     *
     * @param model
     */
    // @ModelAttribute
    // public void addAttributes(Model model) {
    // model.addAttribute("author", "Magical Sam");
    // }

    /**
     * 全局异常捕捉处理
     *
     * @param ex
     * @return
     */
    // @RequestMapping
    @ResponseBody
    @ExceptionHandler({Error.class, Exception.class, Throwable.class})
    public Object errorHandler(HttpServletRequest request, Exception ex) {
        ex.printStackTrace();

        if (ex instanceof LoginException) {
            // return new ResponseEntity<Map<String, Object>>(obj, HttpStatus.UNAUTHORIZED);
            return ResponseUtil.fail401();// 登录过时或未登录
        }

        if (ex instanceof PromiseException) {
            return ResponseUtil.fail400();// 没权限
        }

        if (ex instanceof MethodArgumentNotValidException) {// 业务校验不通过
            MethodArgumentNotValidException exmessage = (MethodArgumentNotValidException) ex;
            return ResponseUtil.fail502(Objects.requireNonNull(exmessage.getBindingResult().getFieldError()).getDefaultMessage());
        }

        if (ex instanceof BusinessException) {// 业务异常
            return ResponseUtil.fail(ex.getMessage());
        }
        if (ex instanceof UnsupportException) {// 业务异常
            return ResponseUtil.fail501(ex.getMessage());
        }

        ex.printStackTrace();
        Map<String, Object> obj = new HashMap<String, Object>();
        obj.put("errno", 502);// 其他错误
        obj.put("data", ex.toString() + "---" + getErrorMsg(ex));
        obj.put("requesturl", request.getRequestURI());

        return obj;
    }

    public String getErrorMsg(Exception ex) {
        StringBuilder sbException = new StringBuilder();
        int i = 0;
        for (StackTraceElement ele : ex.getStackTrace()) {
            sbException.append(ele.getClassName()).append(ele.getMethodName()).append(ele.getFileName()).append(ele.getLineNumber()).append("\n");
            if (i > 4) {
                break;
            }
            i++;
        }
        return sbException.toString();
    }

}
