package com.jzlife.report.schedule;

import com.jzlife.report.dao.reports.SPMapper;
import com.jzlife.report.service.douyin.DouyinPageService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import com.jzlife.report.dao.hmorder.ReportSalseCacheMapper;
import com.jzlife.report.entity.hmorder.ReportSalseCache;
import com.jzlife.report.service.SalesCacheService;
import com.jzlife.report.service.hmorder.DiReportService;
import com.jzlife.report.service.newreport.SalesReportService;
import com.jzlife.report.util.AccountDate;
import com.jzlife.report.util.DateUtils;
import com.jzlife.report.util.JsonUtil;
import com.jzlife.report.util.SortCityUtil;

import lombok.extern.slf4j.Slf4j;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 1，销售报表数据 缓存
 * 2，基础表etl 主要是用crm拉取报表所需的数据
 *
 * @author lu
 *
 */
@Slf4j
@Async
@Component
public class TaskReportData {

	@Autowired
	SalesReportService salesreportService;

	@Autowired
	SalesCacheService salesCacheService;

	@Autowired
	SPMapper spMapper;

	@Autowired
	private DouyinPageService douyinPageService;

	public SimpleDateFormat format = new SimpleDateFormat("yyyy-MM");

	@Scheduled(cron = "0 0 14 * * ?") // 每天10.10执行一次
	public void runDouyinTask() {
		log.info("开始同步抖音页面访问数据-------------------");
		douyinPageService.syncPageData();
		log.info("结束同步抖音页面访问数据-------------------");
	}

	/**
	 * 每日刷新最新的数据
	 */
//	@Scheduled(cron = "0 10 10 * * ?") // 每天10.10执行一次
	public void runTask() {
		log.info("开始生成销售报表数据缓存-------------------");
		String yearMonth = format.format(new Date());
		everyDayCache(yearMonth);
		log.info("生成销售报表数据缓存Ok-------------------");
	}

	/**
	 * 缓存某段时间的销售报表数据
	 *
	 * @param startMonth 2021-11
	 * @param endMonth   2021-12
	 */
	public void insertSalesCache(String startMonth, String endMonth) {
		log.info("缓存某段时间的销售报表数据-------------------");
		List<String> months = AccountDate.getAppointMonths(startMonth, endMonth);
		months.forEach(month -> {
			everyDayCache(month);
		});
		log.info("缓存某段时间的销售报表数据Ok-------------------");
	}

	private void everyDayCache(String yearMonth) {
		List<String> cityList = SortCityUtil.getCity();
		List<String> combinationCity = SortCityUtil.getCombinationCity(cityList);

		combinationCity.forEach(ct -> {
			String arr[] = ct.split(",");
			//单个城市或所有城市的人看
			if (arr.length == 6 || arr.length == 1) {

				Map<String, Object> data = salesreportService.getCompanySalesGmvData(yearMonth, ct);
				ReportSalseCache record = new ReportSalseCache();
				record.setCreateTime(new Date());
				record.setJsonData(JsonUtil.getJson(data));
				record.setParam(SortCityUtil.yearMonthCity(yearMonth, ct));// 城市+年月
				record.setType(SalesCacheService.TYPE_SALES);
				salesCacheService.insertCache(record);
			}

		});
	}


	/**
	 * 每天1.30从生产库 拉取数据表 供报表系统使用
	 */
//	@Scheduled(cron = "0 30 1 * * ?")
	public void runEtlFromCRM() {
		log.info("开始从生产库CRM拉取数据表-------------------");
		spMapper.sp_etl_from_crm();
		log.info("结束从生产库CRM拉取数据表-------------------");
	}
}
