package com.jzlife.report.service;

import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.read.listener.ReadListener;
import com.jzlife.report.dao.nurse.Report14daysCityStackRadioMapper;
import com.jzlife.report.dto.Report14daysCityStackRadio;
import com.jzlife.report.util.SpringContextUtils;
import com.jzlife.report.vo.DaysStackRadioVo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;

import java.util.Date;
import java.util.LinkedList;

@Slf4j
public class ImportExcelListener implements ReadListener<DaysStackRadioVo> {


    private final int batchId;

    private final Report14daysCityStackRadioMapper radioMapper;

    private LinkedList<Report14daysCityStackRadio> tempDatas = new LinkedList<>();

    public ImportExcelListener(int batchId) {
        this.batchId = batchId;
        this.radioMapper = SpringContextUtils.getBeanByClass(Report14daysCityStackRadioMapper.class);
    }

    @Override
    public void invoke(DaysStackRadioVo data, AnalysisContext context) {
        if (data.getDate() == null) {
            return;
        }
        data.setBatchId(batchId);
        data.setImportDate(new Date());
        Report14daysCityStackRadio dbData = new Report14daysCityStackRadio();
        BeanUtils.copyProperties(data, dbData);
        try {
            dbData.setCurrentDayServerCount(Integer.parseInt(data.getCurrentDayServerCount()));
        } catch (NumberFormatException e) {
            log.info("currentDayServerCount is not a number");
            log.info("current data is:{}", data);
        }
        tempDatas.add(dbData);
        int max_temp_count = 1000;
        if (tempDatas.size() >= max_temp_count) {
            radioMapper.batchInsert(tempDatas);
            tempDatas.clear();
        }

    }

    @Override
    public void doAfterAllAnalysed(AnalysisContext context) {
        log.info("read success");
        if (tempDatas.size() > 0) {
            radioMapper.batchInsert(tempDatas);
            tempDatas.clear();
        }
        radioMapper.filterDelete(batchId);

    }
}
