package com.jzlife.report.service.douyin;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.date.LocalDateTimeUtil;
import com.alibaba.fastjson.JSONObject;
import com.jzlife.report.config.DouyinConfig;
import com.jzlife.report.dao.douyin.DouyinPageDataMapper;
import com.jzlife.report.dao.douyin.DouyinPageMapper;
import com.jzlife.report.dao.nurse.JzGoodsMapper;
import com.jzlife.report.dto.douyin.DouYinToken;
import com.jzlife.report.entity.douyin.AppPageLogData;
import com.jzlife.report.entity.douyin.DouyinPage;
import com.jzlife.report.entity.douyin.DouyinPageData;
import com.jzlife.report.util.HttpsUtils;
import lombok.extern.slf4j.Slf4j;

import org.apache.ibatis.annotations.Param;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.io.IOException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.time.ZoneOffset;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Slf4j
@Service
public class DouyinPageService {

    @Autowired
    private DouyinConfig config;

    @Autowired
    private DouyinTokenService douyinTokenService;

    @Resource
    private DouyinPageDataMapper douyinPageDataMapper;

    @Resource
    private DouyinPageMapper douyinPageMapper;
    
    @Autowired
    private JzGoodsMapper goodsMapper;

    public List<DouyinPageData> getList(String start,String end) {
        return douyinPageDataMapper.getList(start, end);
    }


    /**
     * 抖音提示：使用限制
     * 每日 14:00 前更新前一日数据
     * 只能查询昨天以前，且是近90天内的数据
     */
    @Async
    public void syncPageData() {
        //为了防止出错 每次同步2天的数据
        for (int i = 1; i <= 2; i++) {
            long start = LocalDateTimeUtil.beginOfDay(LocalDateTime.now().plusDays(-i)).toInstant(ZoneOffset.ofHours(8)).getEpochSecond();
            long end = LocalDateTimeUtil.endOfDay(LocalDateTime.now().plusDays(-i)).toInstant(ZoneOffset.ofHours(8)).getEpochSecond();
            String date = DateUtil.offsetDay(DateTime.now(), -i).toString("yyyy-MM-dd");
            syncPageData(start, end, date);
        }
    }

    public

    void syncPageData(long start, long end, String date) {
        DouyinPageData douyinPageData = douyinPageDataMapper.getOnePageByDate(date);
        if (douyinPageData != null) return;//表示已经同步过

        //获取所有抖音配置页面
        List<DouyinPage> pageList = douyinPageMapper.getPageList();

        //开始获取页面浏览记录 按天维度
        JSONObject paras = new JSONObject();
        paras.put("start_time", start);
        paras.put("end_time", end);
        try {
            DouYinToken token = douyinTokenService.getToken();
            if (token == null) {
                log.error("douyin token is null");
                return;
            }
            String url = config.getUrl_query_page() + "?start_time=" + start + "&end_time=" + end + "&version_type=online";
            String result = HttpsUtils.doGet(url, token.getAccess_token());

            JSONObject json = JSONObject.parseObject(result);
            if (json.getInteger("err_no").equals(0)) {
                List<DouyinPageData> pageDataList = json.getJSONObject("data").getJSONArray("page_list").toJavaList(DouyinPageData.class);
                //遍历补全页面名称
                for (DouyinPageData pageData : pageDataList) {
                    List<DouyinPage> fp = pageList.stream().filter(item -> item.getPath().equals(pageData.getPage())).collect(Collectors.toList());
                    if (fp.size() > 0)
                        pageData.setPageName(fp.get(0).getName());
                    pageData.setPageDate(new SimpleDateFormat("yyyy-MM-dd").parse(date));
                }
                douyinPageDataMapper.batchInsert(pageDataList);
            }

        } catch (IOException ex) {
            ex.printStackTrace();
        } catch (ParseException e) {
            e.printStackTrace();
        }
    }
    
    public List<AppPageLogData> getPageLog( String start, String end,Integer goodsId,Integer pageId,Integer eventId){
    	return douyinPageDataMapper.getPageLog(start, end,goodsId,pageId,eventId);
    }

    public List<Map<Object, Object>> getDyGoods(){
    	return goodsMapper.getDyGoods();
    }
}
