package com.jzlife.report.service.newreport.impl;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.jzlife.report.dao.hmorder.EtlViewsSalesMapper;
import com.jzlife.report.dao.hmorder.SalesContractMapper;
import com.jzlife.report.dao.hmorder.SalesOrderMapper;
import com.jzlife.report.dto.EchartsLine;
import com.jzlife.report.dto.SalesTargetValue;
import com.jzlife.report.entity.hmorder.ReportSalseCache;
import com.jzlife.report.service.SalesCacheService;
import com.jzlife.report.service.newreport.SalesReportService;
import com.jzlife.report.util.AccountDate;
import com.jzlife.report.util.DateUtil;
import com.jzlife.report.util.DateUtils;
import com.jzlife.report.util.JsonUtil;
import com.jzlife.report.util.SortCityUtil;
import com.jzlife.report.vo.DayReportBaseVo;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.FutureTask;

@Service
public class SalesReportServiceImpl implements SalesReportService {

//    @Autowired
//    EtlGoalMapper etlGoalMapper;

    @Autowired
    EtlViewsSalesMapper etlViewsSalesMapper;

    @Autowired
    SalesOrderMapper saleOrderMapper;

    @Autowired
    SalesCacheService cacheService;

    @Autowired
    SalesContractMapper salesContractMapper;

    @Autowired
    SalesCacheService salesCacheService;

    public static String ALLBUZTYPE = "保洁,保姆,品类,搬家,附加";
    public static String ALLBUZTYPE2 = "保洁,品类";

    /**
     * 公司销售GMV目标
     */
    @Override
    public Map<String, Object> getCompanySalesGmvTarget(String city) {

        int curryear = Calendar.getInstance().get(Calendar.YEAR);
        String startMonth = curryear + "-01";
        String endMonth = curryear + "-12";
//        Map<String, Object> salesMap = etlGoalMapper.salesGmvTarget("销售GMV", startMonth, endMonth);
        BigDecimal salesGmvTarget = saleOrderMapper.salesGmvTarget(city, null, "销售GMV", startMonth, endMonth);
        BigDecimal complate = saleOrderMapper.countSalesYear(city, curryear + "");

        double bl = 0.0;
        String b = "0";
        if (salesGmvTarget != null && salesGmvTarget != null
                && salesGmvTarget.intValue() != 0 && salesGmvTarget.intValue() != 0 && complate != null) {
            bl = complate.doubleValue() / salesGmvTarget.doubleValue();
        }

//        		new BigDecimal(
//                salesMap.get("completionRatio") == null ? "0" : salesMap.get("completionRatio").toString())
//                .doubleValue();// 完成比例
        if (bl > 0.0) {
            bl = bl * 100;
            b = Double.toString(bl).substring(0, Double.toString(bl).indexOf(".") + 3);
        }

        Map<String, Object> data = new HashMap<String, Object>();
        data.put("totalTargetNumber", salesGmvTarget == null ? 0 : salesGmvTarget);// 当年总目标数
        data.put("completedNumber", complate == null ? 0 : complate.doubleValue());// 已完成数
        data.put("completionRatio", b + "%");// 完成比例
        data.put("startYm", curryear + "年01月");// 开始年月
        data.put("endYm", curryear + "年12月");// 结束年月

        return data;
    }

    /**
     * GMV数据 yearMonth 2021-01
     */
    @Override
    public Map<String, Object> getCompanySalesGmvData(String yearMonth, String city) {

    	//不是当月查缓存
    	if(!DateUtils.checkThisTime(yearMonth)) {
    		ReportSalseCache typeCache = cacheService.getTypeCache(SalesCacheService.TYPE_SALES,
    				SortCityUtil.yearMonthCity(yearMonth, city));
    		if (null != typeCache && typeCache.getJsonData() != null) {
    			Map<String, Object> cachdata = JsonUtil.getEntity(typeCache.getJsonData(), Map.class);
    			if (null != cachdata) {
    				return cachdata;
    			}
    		}
    	}

        ExecutorService executorService = Executors.newFixedThreadPool(10);

        Callable<Map> gmvDataSale = () -> getGMVData(yearMonth, city);// 销售gmv
        Callable<Map> gmvDataService = () -> getServiceGMVData(yearMonth, city);// 服务gmv
        Callable<Map> buyAgenDatas = () -> BuyAgenDatas(yearMonth, city);// 复购情况
        Callable<List> buzLineProgress = () -> getBuzLineProgress(yearMonth, city);// 当月各业务线进度看板
        Callable<List> baomuProgress = () -> baomuProgress(yearMonth, city);// 保姆进度进度看板

        FutureTask<Map> task1 = new FutureTask<>(gmvDataSale);
        FutureTask<Map> task2 = new FutureTask<>(gmvDataService);
        FutureTask<Map> task3 = new FutureTask<>(buyAgenDatas);
        FutureTask<List> task4 = new FutureTask<>(buzLineProgress);
        FutureTask<List> task5 = new FutureTask<>(baomuProgress);
        executorService.submit(task1);
        executorService.submit(task2);
        executorService.submit(task3);
        executorService.submit(task4);
        executorService.submit(task5);

        Map<String, Object> data = new HashMap<String, Object>();

        try {
            data.put("gmvDataSale", task1.get());// 销售gmv
            data.put("gmvDataService", task2.get());// 服务gmv
            data.put("buyAgenDatas", task3.get());// 复购情况
            data.put("buzLineProgress", task4.get());// 当月各业务线进度看板
            data.put("baomuProgress", task5.get());// 保姆进度进度看板

        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            executorService.shutdown();
        }

//		Map<String, Object> gmvDataSale = getGMVData(yearMonth);
//		Map<String, Object> gmvDataService = getServiceGMVData(yearMonth);
//		Map<String, Object> buyAgenDatas = BuyAgenDatas(yearMonth,
//				Double.parseDouble(gmvDataSale.get("GMV").toString()) * 10000);
//		List<SalesTargetValue> buzLineProgress = getBuzLineProgress(yearMonth);
//		List<SalesTargetValue> baomuProgress = baomuProgress(null,yearMonth);

//		Map<String, Object> data = new HashMap<String, Object>();
//		data.put("gmvDataSale", gmvDataSale);// 销售gmv
//		data.put("gmvDataService", gmvDataService);// 服务gmv
//		data.put("buyAgenDatas", buyAgenDatas);// 复购情况
//		data.put("buzLineProgress", buzLineProgress);// 当月各业务线进度看板
//		data.put("baomuProgress", buzLineProgress);// 保姆进度进度看板

        //不是当月缓存起来
        if(!DateUtils.checkThisTime(yearMonth)&&data.size()>0) {
			ReportSalseCache record = new ReportSalseCache();
			record.setCreateTime(new Date());
			record.setJsonData(JsonUtil.getJson(data));
			record.setParam(SortCityUtil.yearMonthCity(yearMonth, city));// 城市+年月
			record.setType(SalesCacheService.TYPE_SALES);
			salesCacheService.insertCache(record);
        }

        return data;
    }

    public List<SalesTargetValue> baomuProgress(String yearMonth, String citys) {
        long start = System.currentTimeMillis();
        Map<String, String> type = new LinkedHashMap<String, String>();
        type.put("合计", "");
        type.put("钟点工", "钟点工");
        type.put("住家保姆", "住家保姆");
        type.put("白班保姆", "白班保姆");
        type.put("代班", "代班保姆-钟点工,代班保姆-保姆");
        type.put("月嫂", "月嫂");

        String sy = shangyue(yearMonth);// 上月
        String mh = yearMonth + "," + shangyue(yearMonth);

        List<SalesTargetValue> listdata = new ArrayList<SalesTargetValue>();

        type.forEach((k, v) -> {

            if (k.equals("合计")) {
                String tempmh = yearMonth + "-01";
                if (DateUtil.dateToString(new Date(), DateUtil.DATE_FORMAT_YM).equals(yearMonth)) {
                    tempmh = DateUtil.getCurrentDate();
                }

                String dyarr[] = xdshangyue(1, tempmh).split(",");// 当月日期
                String syarr[] = xdshangyue(2, tempmh).split(",");// 上月日期

                // 成单率
                BigDecimal dycdl = salesContractMapper.chengdanlv(citys, dyarr[0], dyarr[1]);
                BigDecimal sycdl = salesContractMapper.chengdanlv(citys, syarr[0], syarr[1]);

                // 上月成单率
                BigDecimal dypjgs = salesContractMapper.pingjunqdsc(citys, dyarr[0], dyarr[1]);
                BigDecimal sypjgs = salesContractMapper.pingjunqdsc(citys, syarr[0], syarr[1]);

                SalesTargetValue cdl30 = new SalesTargetValue("合计", "30天成单率", dycdl, sycdl);
                SalesTargetValue pjqdsc = new SalesTargetValue("合计", "平均签单时长", dypjgs, sypjgs);

                listdata.add(cdl30);
                listdata.add(pjqdsc);
            }

            List<Map<String, Object>> clueSignCount = salesContractMapper.getClueSignCount(citys, v, mh);
            Map<String, Object> dyData = getMonthData(clueSignCount, yearMonth);
            Map<String, Object> syData = getMonthData(clueSignCount, sy);

            List<Map<String, Object>> refundOrder = salesContractMapper.getRefundOrder(citys, v, mh);
            Map<String, Object> dyrfData = getMonthData(refundOrder, yearMonth);
            Map<String, Object> syrfData = getMonthData(refundOrder, sy);

            SalesTargetValue sfxs = new SalesTargetValue(k, "分配线索数", dyData.get("clue_count"),
                    syData.get("clue_count"));
            SalesTargetValue qds = new SalesTargetValue(k, "签单数", dyData.get("sign_count"), syData.get("sign_count"));
            SalesTargetValue tds = new SalesTargetValue(k, "退单数", dyrfData.get("refund_count"),
                    syrfData.get("refund_count"));
            SalesTargetValue htsk = new SalesTargetValue(k, "合同收款", dyrfData.get("sum_pay"), syrfData.get("sum_pay"));

            listdata.add(sfxs);
            listdata.add(qds);
            listdata.add(tds);
            listdata.add(htsk);
        });
        System.out.println("第5个耗时:" + (System.currentTimeMillis() - start));
        return listdata;
    }

    public Map<String, Object> getMonthData(List<Map<String, Object>> list, String yearMonth) {
        for (Map<String, Object> s : list) {
            if (s.get("dm") != null && s.get("dm").toString().equals(yearMonth)) {
                return s;
            }
        }
        return new HashMap<String, Object>();
    }

    // 自然月的上月
    public String shangyue(String yearMonth) {
        String[] ym = yearMonth.split("-");
        int year = Integer.parseInt(ym[0]);
        int month = Integer.parseInt(ym[1]);
//		String months = month < 10 ? ("0" + month) : month + "";

        // 上月 ----------------
        int lastmonth2 = month - 1;
        int tempyear = year;
        if (lastmonth2 == 0) {
            lastmonth2 = 12;
            tempyear = tempyear - 1;
        }
        String mh = lastmonth2 + "";
        if (lastmonth2 < 10) {
            mh = "0" + mh;
        }
        return tempyear + "-" + mh;
    }

    /**
     * 成单率和签单时常的当月,上月
     *
     * @param t    1当月，2上月
     * @param date yyyy-mm-dd
     * @return
     */
    public static String xdshangyue(int t, String date) {
        // 判断单日是否1号、

        boolean firstDayOfMonth = DateUtil.isFirstDayOfMonth(DateUtil.stringToDate(date));
        // 如果是第一天，
        if (firstDayOfMonth) {
            if (t == 1) {
                // 拿上个月的第一天和最后一天
                Date start = DateUtil.addMonth(new Date(), -1);// 实际是上月
                start = DateUtil.getFirstDayOfMonth(start);// 月的第一天
                Date end = DateUtil.getLastDayOfMonth(start);// 月的最后一天
                return DateUtil.dateToString(start) + "," + DateUtil.dateToString(end);
            } else {//(t == 2)
                // 拿上个月的第一天和最后一天
                Date start = DateUtil.addMonth(new Date(), -2);// 实际是上上月
                start = DateUtil.getFirstDayOfMonth(start);// 月的第一天
                Date end = DateUtil.getLastDayOfMonth(start);// 月的最后一天
                return DateUtil.dateToString(start) + "," + DateUtil.dateToString(end);
            }

        } else {
            if (t == 1) {
                // 截止昨天，拿前31天到前1天
                String start = DateUtil.getPreDateByDate(date, -31);
                String end = DateUtil.getPreDateByDate(date, -1);
                return start + "," + end;
            } else {//(t == 2)
                // 拿前62天到前1天
                String start = DateUtil.getPreDateByDate(date, -61);
                String end = DateUtil.getPreDateByDate(date, -31);
                return start + "," + end;
            }
        }
    }


    /**
     * 当月各业务线进度看板
     *
     * @param yearMonth 2021-12
     * @return
     */
    public List<SalesTargetValue> getBuzLineProgress(String yearMonth, String city) {
        long start = System.currentTimeMillis();
        List<String> levelOnelist = new ArrayList<String>();
        levelOnelist.add("保洁");
        levelOnelist.add("品类");
        levelOnelist.add("保姆");
        levelOnelist.add("搬家");

        // 去年-----------
        String[] ym = yearMonth.split("-");
        int year = Integer.parseInt(ym[0]);
        int month = Integer.parseInt(ym[1]);
        String months = month < 10 ? ("0" + month) : month + "";

        String hbsy = shangyue(yearMonth);// 环比上月
        String tbqn = year - 1 + "-" + months;// 同比去年

        Map<String, String> type = new HashMap<String, String>();
        type.put("当月累计", yearMonth);
        type.put("完成进度", yearMonth);// 也是当月
        type.put("环比上月", hbsy);
        type.put("同比去年", tbqn);

        List<String> dayList = new ArrayList<String>();
        dayList.add(yearMonth);
        dayList.add(hbsy);
        dayList.add(tbqn);

        // -------------------处理----------------------------
        List<Map<String, Object>> gmvSales = saleOrderMapper.salesGmvqushiMonth(null, city, null, null, dayList);
        List<Map<String, Object>> gmvService = saleOrderMapper.serviceGmvdanliangMonth(city, null, dayList);

        List<SalesTargetValue> listdata = new ArrayList<SalesTargetValue>();

        for (String lvone : levelOnelist) {
            type.forEach((k, yearm) -> {
                Map<String, Object> sales = getLevelOneMonth(gmvSales, lvone, yearMonth);// 当月的总销售
                Map<String, Object> service = getLevelOneMonth(gmvService, lvone, yearMonth);// 当月的总服务

                SalesTargetValue sv = new SalesTargetValue();
                sv.setBuzLine(lvone);
                sv.setTerm(k);

                if (null != sales && service != null) {
                    BigDecimal salessz = new BigDecimal(sales.get("sz").toString());// 当月销售
                    BigDecimal servicesz = new BigDecimal(service.get("sfje").toString());// 当月服务
                    if (k.equals("当月累计")) {
                        sv.setSales(salessz);
                        if (lvone.equals("品类") || lvone.equals("保洁")) {
                            sv.setService(servicesz);
                        }
                    }
                    if (k.equals("完成进度")) {
//						int curryear = Calendar.getInstance().get(Calendar.YEAR);
//						String startMonth = curryear + "-01";
//						String endMonth = curryear + "-12";

                        BigDecimal salesTarget = saleOrderMapper.salesGmvTarget(city, lvone, "销售GMV", yearMonth,
                                yearMonth);
                        salesTarget = salesTarget == null ?BigDecimal.ZERO : salesTarget;
                        if (salessz.intValue() == 0 || salesTarget.intValue() == 0) {
                            sv.setSales("0" + "%");
                        } else {
                            BigDecimal aa = salessz.divide(salesTarget, 4, BigDecimal.ROUND_HALF_UP)
                                    .multiply(new BigDecimal(100));
                            sv.setSales(setScale(aa) + "%");
                        }

                        if (lvone.equals("品类") || lvone.equals("保洁")) {
                            BigDecimal serviceTarget = saleOrderMapper.salesGmvTarget(city, lvone, "服务GMV", yearMonth,
                                    yearMonth);
                            serviceTarget = serviceTarget == null ? BigDecimal.ZERO : serviceTarget;
                            //						data.put("currentMonthTarget", setScale(aa) + "%");// 当月已完成总目标 当月已完成除以当年总目标
                            if (servicesz.intValue() == 0 || serviceTarget.intValue() == 0) {
                                sv.setService("0" + "%");
                            } else {
                                BigDecimal bb = servicesz.divide(serviceTarget, 4, BigDecimal.ROUND_HALF_UP)
                                        .multiply(new BigDecimal(100));
                                sv.setService(setScale(bb) + "%");
                            }
                        }
                    }
                    if (k.equals("环比上月") || k.equals("同比去年")) {
                        Map<String, Object> salesh = getLevelOneMonth(gmvSales, lvone, yearm);
                        Map<String, Object> serviceh = getLevelOneMonth(gmvService, lvone, yearm);

                        BigDecimal hbsales = salesh == null ? BigDecimal.ZERO
                                : new BigDecimal(salesh.get("sz").toString());// 上月,去年 销售
                        BigDecimal hbservice = serviceh == null ? BigDecimal.ZERO
                                : new BigDecimal(serviceh.get("sfje").toString());// 上月，去年 服务

                        BigDecimal bb = BigDecimal.ZERO;
                        if (hbsales != null && salessz.intValue() != 0 && hbsales.intValue() != 0) {
                            bb = setScale((salessz.subtract(hbsales)).divide(hbsales, 4, BigDecimal.ROUND_HALF_UP)
                                    .multiply(new BigDecimal(100)));
                        }

                        BigDecimal cc = BigDecimal.ZERO;
                        if (hbservice != null && servicesz.intValue() != 0 && hbservice.intValue() != 0) {
                            cc = setScale((servicesz.subtract(hbservice)).divide(hbservice, 4, BigDecimal.ROUND_HALF_UP)
                                    .multiply(new BigDecimal(100)));
                        }
                        sv.setSales(bb + "%");
                        if (lvone.equals("品类") || lvone.equals("保洁")) {
                            sv.setService(cc + "%");
                        }

                    }
                }
                listdata.add(sv);
            });

        }

        System.out.println("第4个耗时:" + (System.currentTimeMillis() - start));
        return listdata;
    }

    public Map<String, Object> getLevelOneMonth(List<Map<String, Object>> datalist, String levelOne, String month) {
        for (Map<String, Object> d : datalist) {
            if (d.get("levelone") == null || d.get("dy") == null) {
                continue;
            }
            if (d.containsKey("levelone") && d.containsKey("dy") && d.get("levelone").equals(levelOne)
                    && d.get("dy").equals(month)) {
                return d;
            }
        }

        return null;
    }

    /**
     * 实销额和单量
     */
    @Override
    public Map<String, Object> getCompanySalesbrokenLine(String channel, String cityName, String businessTypeName, String businessLine,
                                                         Integer timeType, String startTime, String endTime, String startMonth, String endMonth) {
//        if (!StringUtils.isBlank(businessLine)) {
//            businessTypeName = businessLine;
//            businessLine = null;
//        }
        Map<String, Object> data = new HashMap<String, Object>();
        if (timeType == 1) {
            // 天
            List<String> dayList = AccountDate.getEveryday(startTime, endTime);
            List<EchartsLine> echList = new ArrayList<EchartsLine>();

            List<Map<String, Object>> gmvDay = saleOrderMapper.salesGmvqushiDay(channel, cityName, businessTypeName,businessLine, dayList);

//			List<String> collect = dayList.stream().map(d->d).collect(Collectors.toList());

            // 实销额，退款额
            if (true) {

                List<Object> tempListTK = new ArrayList<Object>();

                for (String day : dayList) {
                    BigDecimal tk = BigDecimal.ZERO;
                    for (Map<String, Object> e : gmvDay) {
                        if (e.get("dy").toString().equals(day) && e.get("tk") != null && !e.get("tk").toString().equals("0")) {
                            tk = tk.add(new BigDecimal(e.get("tk").toString()));
                        }
                    }
                    tempListTK.add(setScale(tk));
                }

                for (String bu : businessTypeName.split(",")) {
                    List<Object> tempListSX = new ArrayList<Object>();
                    for (String day : dayList) {
                    	boolean boo = true;
                        for (Map<String, Object> e : gmvDay) {
                            if (e.get("dy").toString().equals(day) && e.get("sz") != null
                                    && e.get("levelone").toString().equals(bu)
                                    && !e.get("sz").toString().equals("0")) {
                                BigDecimal sx = new BigDecimal(e.get("sz").toString());
                                tempListSX.add(setScale(sx));
                                boo = false;
                            }
                        }
                        if (boo) {
                            tempListSX.add("");
                        }
                    }
                    EchartsLine el = new EchartsLine("bar");
                    el.setData(tempListSX);
                    el.setStack("Total");
                    el.setName(bu + "实销额");
                    echList.add(el);
                }

                EchartsLine el2 = new EchartsLine("bar");
                el2.setData(tempListTK);
                el2.setStack("Total");
                el2.setName("退款额");
                echList.add(el2);
            }

            // 其他折线
            for (String bu : businessTypeName.split(",")) {
                EchartsLine el = new EchartsLine("line");
                List<Object> tempList = new ArrayList<Object>();

                for (String day : dayList) {
                    boolean boo = true;
                    for (Map<String, Object> e : gmvDay) {
                        if (e.get("dy").toString().equals(day) && e.get("levelone").toString().equals(bu)
                                && e.get("xsdl") != null && !e.get("xsdl").toString().equals("0")) {
                            tempList.add(e.get("xsdl"));
                            boo = false;
                        }
                    }
                    if (boo) {
                        tempList.add("");
                    }
                }
                el.setName(bu);
                el.setData(tempList);
                echList.add(el);
            }

            // 每日的总单量(总单量不受商品属性条件限制 businnessLine)
            if (true) {
                List<Map<String, Object>> gmvDayCount = saleOrderMapper.salesGmvqushiDay(channel, cityName, ALLBUZTYPE, null,dayList);
                List<Object> tempList = new ArrayList<Object>();
                List<Object> tkList = new ArrayList<Object>();
                for (String day : dayList) {
                    BigDecimal v = BigDecimal.ZERO;
                    BigDecimal tk = BigDecimal.ZERO;
                    for (Map<String, Object> e : gmvDayCount) {
                        if (e.get("dy").toString().equals(day) && e.get("xsdl") != null) {
                            v = v.add(new BigDecimal(e.get("xsdl").toString()));
                        }
                        if (e.get("dy").toString().equals(day) && e.get("tkdl") != null) {
                            tk = tk.add(new BigDecimal(e.get("tkdl").toString()));
                        }
                    }
                    tempList.add(setScale(v));
                    tkList.add(setScale(tk));
                }
                EchartsLine el = new EchartsLine("line");
                el.setData(tempList);
                el.setName("每日销售总单量");
                echList.add(el);

                el = new EchartsLine("line");
                el.setData(tkList);
                el.setName("每日退款单量");
                echList.add(el);
            }

            data.put("series", echList);
            data.put("days", dayList);

        } else {
            // 月
            List<String> months = AccountDate.getAppointMonths(startMonth, endMonth);
            List<EchartsLine> echList = new ArrayList<EchartsLine>();

            List<Map<String, Object>> gmvDay = saleOrderMapper.salesGmvqushiMonthBj(channel, cityName, businessTypeName,
                    businessLine, months);

            // 每月的实销额，退款额
            if (true) {
                List<Object> tempListTK = new ArrayList<Object>();

                for (String day : months) {
                    BigDecimal tk = BigDecimal.ZERO;
                    for (Map<String, Object> e : gmvDay) {
                        if (e.get("dy").toString().equals(day) && e.get("tk") != null && !e.get("tk").toString().equals("0")) {
                            tk = tk.add(new BigDecimal(e.get("tk").toString()));
                        }
                    }
                    tempListTK.add(setScale(tk));

                }
                for (String bu : businessTypeName.split(",")) {
                    List<Object> tempListSX = new ArrayList<Object>();
                    for (String day : months) {
                    	boolean boo = true;
                        for (Map<String, Object> e : gmvDay) {
                            if (e.get("dy").toString().equals(day) && e.get("sz") != null
                                    && e.get("levelone").toString().equals(bu) && !e.get("sz").toString().equals("0")) {
                                BigDecimal sx = new BigDecimal(e.get("sz").toString());
                                tempListSX.add(setScale(sx));
                                boo = false;
                            }
                        }
                        if (boo) {
                            tempListSX.add("");
                        }
                    }
                    EchartsLine el = new EchartsLine("bar");
                    el.setData(tempListSX);
                    el.setStack("Total");
                    el.setName(bu + "实销额");
                    echList.add(el);
                }

                EchartsLine el2 = new EchartsLine("bar");
                el2.setData(tempListTK);
                el2.setStack("Total");
                el2.setName("退款额");
                echList.add(el2);
            }

            // 其他折线
            for (String bu : businessTypeName.split(",")) {
                List<Object> tempList = new ArrayList<Object>();

                for (String day : months) {
                    boolean boo = true;
                    for (Map<String, Object> e : gmvDay) {
                        if (e.get("dy").toString().equals(day) && e.get("levelone").toString().equals(bu)
                                && e.get("xsdl") != null) {
                            tempList.add(e.get("xsdl"));
                            boo = false;
                        }
                    }
                    if (boo) {
                        tempList.add("");
                    }
                }
                EchartsLine el = new EchartsLine("line");
                el.setName(bu);
                el.setData(tempList);
                echList.add(el);
            }

            // 每月的 总单量
            if (true) {
                List<Map<String, Object>> gmvDayCount = saleOrderMapper.salesGmvqushiMonth(channel, cityName, null, ALLBUZTYPE,
                        months);
                List<Object> tempList = new ArrayList<Object>();
                List<Object> tkList = new ArrayList<Object>();

                for (String day : months) {
                    BigDecimal v = BigDecimal.ZERO;
                    BigDecimal tk = BigDecimal.ZERO;
                    for (Map<String, Object> e : gmvDayCount) {
                        if (e.get("dy").toString().equals(day) && e.get("xsdl") != null) {
                            v = v.add(new BigDecimal(e.get("xsdl").toString()));
                        }
                        if (e.get("dy").toString().equals(day) && e.get("tkdl") != null) {
                            tk = tk.add(new BigDecimal(e.get("tkdl").toString()));

                        }
                    }
                    tempList.add(setScale(v));
                    tkList.add(setScale(tk));
                }
                EchartsLine el = new EchartsLine("line");
                el.setData(tempList);
                el.setName("每月销售总单量");
                echList.add(el);

                el = new EchartsLine("line");
                el.setData(tkList);
                el.setName("每月退款单量");
                echList.add(el);
            }
            data.put("series", echList);
            data.put("days", months);
        }

        return data;
    }

    /**
     * 服务GMV与单量
     */
    @Override
    public Map<String, Object> getOrderNumberchart(String cityName, String businessTypeName, Integer timeType,
                                                   String startTime, String endTime, String startMonth, String endMonth) {
        Map<String, Object> data = new HashMap<String, Object>();
        if (timeType == 1) {
            // 天
            List<String> dayList = AccountDate.getEveryday(startTime, endTime);
            List<EchartsLine> echList = new ArrayList<EchartsLine>();

            List<Map<String, Object>> gmvDay = saleOrderMapper.serviceGmvdanliangDay(cityName, businessTypeName,
                    dayList);

            // 实付金额，派遣单金额
            if (true) {
                List<Object> tempListSF = new ArrayList<Object>();
                List<Object> tempListPQ = new ArrayList<Object>();

                for (String day : dayList) {
                    BigDecimal sf = BigDecimal.ZERO;
                    BigDecimal pq = BigDecimal.ZERO;

                    for (Map<String, Object> e : gmvDay) {
                        if (e.get("dy").toString().equals(day) && e.get("sfje") != null) {
                            sf = sf.add(new BigDecimal(e.get("sfje").toString()));
                        }
                        if (e.get("dy").toString().equals(day) && e.get("pqdje") != null) {
                            pq = pq.add(new BigDecimal(e.get("pqdje").toString()));
                        }
                    }
                    tempListSF.add(setScale(sf));
                    tempListPQ.add(setScale(pq));
                }
                EchartsLine el = new EchartsLine("bar");
                el.setData(tempListSF);
                el.setStack("Total");
                el.setName("实付金额");
                echList.add(el);

                EchartsLine el2 = new EchartsLine("bar");
                el2.setData(tempListPQ);
                el2.setStack("Total");
                el2.setName("派遣单金额");
                echList.add(el2);
            }

            for (String bu : businessTypeName.split(",")) {
                EchartsLine el = new EchartsLine("line");
                List<Object> tempList = new ArrayList<Object>();

                for (String day : dayList) {
                    boolean boo = true;
                    for (Map<String, Object> e : gmvDay) {
                        if (e.get("dy").toString().equals(day) && e.get("levelone").toString().equals(bu)
                                && e.get("zdl") != null) {
                            tempList.add(e.get("zdl"));
                            boo = false;
                        }
                    }
                    if (boo) {
                        tempList.add(0);
                    }
                }

                el.setName(bu);
                el.setData(tempList);
                echList.add(el);
            }

            // 每日的 总单量
            if (true) {
                List<Map<String, Object>> gmvDayCount = saleOrderMapper.serviceGmvdanliangDay(cityName, ALLBUZTYPE2,
                        dayList);
                List<Object> tempList = new ArrayList<Object>();

                for (String day : dayList) {
                    BigDecimal v = BigDecimal.ZERO;
                    for (Map<String, Object> e : gmvDayCount) {
                        if (e.get("dy").toString().equals(day) && e.get("zdl") != null) {
                            v = v.add(new BigDecimal(e.get("zdl").toString()));

                        }
                    }
                    tempList.add(setScale(v));

                }
                EchartsLine el = new EchartsLine("line");
                el.setData(tempList);
                el.setName("每日总单量");
                echList.add(el);
            }

            data.put("series", echList);
            data.put("days", dayList);

        } else {
            // 月
            List<String> months = AccountDate.getAppointMonths(startMonth, endMonth);
            List<EchartsLine> echList = new ArrayList<EchartsLine>();

            List<Map<String, Object>> gmvDay = saleOrderMapper.serviceGmvdanliangMonth(cityName, businessTypeName,
                    months);

            // 实付金额，派遣单金额
            if (true) {
                List<Object> tempListSF = new ArrayList<Object>();
                List<Object> tempListPQ = new ArrayList<Object>();

                for (String day : months) {
                    BigDecimal sf = BigDecimal.ZERO;
                    BigDecimal pq = BigDecimal.ZERO;

                    for (Map<String, Object> e : gmvDay) {
                        if (e.get("dy").toString().equals(day) && e.get("sfje") != null) {
                            sf = sf.add(new BigDecimal(e.get("sfje").toString()));
                        }
                        if (e.get("dy").toString().equals(day) && e.get("pqdje") != null) {
                            pq = pq.add(new BigDecimal(e.get("pqdje").toString()));
                        }
                    }
                    tempListSF.add(setScale(sf));
                    tempListPQ.add(setScale(pq));
                }
                EchartsLine el = new EchartsLine("bar");
                el.setData(tempListSF);
                el.setStack("Total");
                el.setName("实付金额");
                echList.add(el);

                EchartsLine el2 = new EchartsLine("bar");
                el2.setData(tempListPQ);
                el2.setStack("Total");
                el2.setName("派遣单金额");
                echList.add(el2);
            }
            for (String bu : businessTypeName.split(",")) {
                EchartsLine el = new EchartsLine("line");
                List<Object> tempList = new ArrayList<Object>();

                for (String day : months) {
//                     Double gmvDay = etlViewsSalesMapper.salesGmvdanliangMonth(cityName, getTypeValue(bu), mt);
                    boolean boo = true;
                    for (Map<String, Object> e : gmvDay) {
                        if (e.get("dy").toString().equals(day) && e.get("levelone").toString().equals(bu)
                                && e.get("zdl") != null) {
                            tempList.add(e.get("zdl"));
                            boo = false;
                        }
                    }
                    if (boo) {
                        tempList.add(0);
                    }
                }

                el.setName(bu);
                el.setData(tempList);
                echList.add(el);
            }

            // 每月的 总单量
            if (true) {
                List<Map<String, Object>> gmvDayCount = saleOrderMapper.serviceGmvdanliangMonth(cityName, ALLBUZTYPE2,
                        months);
                List<Object> tempList = new ArrayList<Object>();

                for (String day : months) {
                    BigDecimal v = BigDecimal.ZERO;
                    for (Map<String, Object> e : gmvDayCount) {
                        if (e.get("dy").toString().equals(day) && e.get("zdl") != null) {
                            v = v.add(new BigDecimal(e.get("zdl").toString()));

                        }
                    }
                    tempList.add(setScale(v));

                }
                EchartsLine el = new EchartsLine("line");
                el.setData(tempList);
                el.setName("每月总单量");
                echList.add(el);
            }

            data.put("series", echList);
            data.put("days", months);

        }
        return data;
    }

    @Override
    public PageInfo<Map<String, Object>> bdInvertDetail(DayReportBaseVo vo) {
        if (StringUtils.isEmpty(vo.getCity())) {
            vo.setCity(null);
        }
        PageHelper.startPage(vo.getStartPage(), vo.getLimit());
        List<Map<String, Object>> mapList = saleOrderMapper.bdInvertDetail(vo.getStartTime(), vo.getEndTime(),
                vo.getCity());
        return new PageInfo<>(mapList);
    }

    @Override
    public List<Map<String, Object>> bdInvertDetailNoPage(DayReportBaseVo vo) {
        if (StringUtils.isEmpty(vo.getCity())) {
            vo.setCity(null);
        }
        return saleOrderMapper.bdInvertDetail(vo.getStartTime(), vo.getEndTime(),
                vo.getCity());
    }

    /**
     * 复购情况
     *
     * @param yearMonth
     * @param city       城市
     */
    private Map<String, Object> BuyAgenDatas(String yearMonth, String city) {
        long start = System.currentTimeMillis();
        BigDecimal kaidan = saleOrderMapper.kaidan(city, yearMonth);
        kaidan = kaidan == null ? BigDecimal.ZERO : kaidan;
        // 当月复购金额-------------------------------------------------------------------
        BigDecimal buyAgainAmounts = saleOrderMapper.countfugou(city, DateUtils.qytdyddyt(DateUtils.isThisTime(yearMonth)),DateUtils.getFrontDay(DateUtils.isThisTime(yearMonth)));
        Double buyAgainAmount = buyAgainAmounts == null ? 0.0 : buyAgainAmounts.doubleValue();


        // 上月复购金额-------------------------------------------------------------------
        BigDecimal lastbuyAgainAmounts = saleOrderMapper.countfugou(city, DateUtils.getqytsydyt(DateUtils.isThisTime(yearMonth)), DateUtils.getqytsydr(DateUtils.isThisTime(yearMonth)));
        double lastbuyAgainAmount = lastbuyAgainAmounts == null ? 0.0 : lastbuyAgainAmounts.doubleValue();

        Double thanlastbA = buyAgainAmount - lastbuyAgainAmount;// 对比上月增多少（本月-上月）
        double thanlastbARatio = ((buyAgainAmount - lastbuyAgainAmount) / lastbuyAgainAmount) * 100;// 对比上月增多少（本月-上月）/上月
        if (buyAgainAmount == 0.0 || lastbuyAgainAmount == 0.0) {
            thanlastbARatio = 0.0;
        }

        // 线索转化率---------------------------------------------------
        BigDecimal clueZhl = saleOrderMapper.clueZhl(city, yearMonth);

        // 保洁，品类
        // 新客率-------------------------------------------------------------------------------------------
        Map<String, BigDecimal> newCustomerRate = saleOrderMapper.newCustomerRate(city, yearMonth);
        Double bj = 0.0;
        Double pl = 0.0;
        if (null != newCustomerRate) {
            bj = newCustomerRate.get("bj") == null ? 0.0 : newCustomerRate.get("bj").doubleValue();
            pl = newCustomerRate.get("pl") == null ? 0.0 : newCustomerRate.get("pl").doubleValue();
        }

        Date addDay = DateUtil.addDay(new Date(), -30);
        String beforeMonth = DateUtil.dateToString(addDay);

        Map<String, Object> buyAgain = new HashMap<String, Object>();

        double buyAgainRatio = 0.0;//开单金额/(销售额-退款额)
        if (buyAgainAmount != 0.0 && kaidan.intValue() != 0) {
            buyAgainRatio = buyAgainAmount / kaidan.doubleValue();
        }

        buyAgain.put("buyAgainAmount", tranMoney(buyAgainAmount));// 当月复购金额
        buyAgain.put("buyAgainRatio", subLast(buyAgainRatio * 100) + "%");// 当月复购占比
        buyAgain.put("thanlastbA", tranMoney(thanlastbA));// 对比上月增多少
        buyAgain.put("thanlastbARatio", subLast(thanlastbARatio) + "%");// 对比上月增多少比列
        buyAgain.put("baojeixinke", subLast(bj * 100) + "%");// 保洁新客率
        buyAgain.put("pingleixinke", subLast(pl * 100) + "%");// 品类新客率
        String xiansuozhuanhua = "0";
        if (null != clueZhl && clueZhl.doubleValue() > 0) {
            xiansuozhuanhua = subLast(clueZhl.doubleValue() * 100);
        }
        buyAgain.put("xiansuozhuanhua", xiansuozhuanhua + "%");// 保姆线索转化率
        buyAgain.put("chenjikehu", saleOrderMapper.countCustomerQuiet(city, beforeMonth));// 沉寂客户数

        System.out.println("第三个耗时:" + (System.currentTimeMillis() - start));
        return buyAgain;
    }

    /**
     * 将元转换成万元
     *
     * @return BigDecimal
     */
    private BigDecimal tranMoney(Double money) {
        return new BigDecimal(money).divide(new BigDecimal(10000), 2, RoundingMode.HALF_UP);
    }

    private BigDecimal tranMoney2(BigDecimal money) {
        return money.divide(new BigDecimal(10000), 2, RoundingMode.HALF_UP);
    }

    /**
     * gmv获取
     *
     * @param yearMonth 年月 2021-06
     * @return
     */
    private Map<String, Object> getGMVData(String yearMonth, String city) {
        long start = System.currentTimeMillis();
//        String[] ym = yearMonth.split("-");
//        int year = Integer.parseInt(ym[0]);
//        int month = Integer.parseInt(ym[1]);
//        String months = month < 10 ? ("0" + month) : month + "";

        // 当年---------------------------------------------------------
//        int curryear = Calendar.getInstance().get(Calendar.YEAR);
//        String startMonth = curryear + "-01";
//        String endMonth = curryear + "-12";
        BigDecimal totalTargetNumber = saleOrderMapper.salesGmvTarget(city, null, "销售GMV", yearMonth, yearMonth);// startMonth,
        // endMonth

        // 当月---------------------------------------------------------
        Map<String, BigDecimal> countSalesYm = saleOrderMapper.countSalesYm(city, DateUtils.qytdyddyt(DateUtils.isThisTime(yearMonth)), DateUtils.getFrontDay(DateUtils.isThisTime(yearMonth)));
        BigDecimal sale = countSalesYm == null ? BigDecimal.ZERO : countSalesYm.get("sales");// saleOrderMapper.countSalesYm(yearMonth);
        BigDecimal saleNumber = BigDecimal.ZERO;
        if (sale != null) {
            saleNumber = sale;// 当月已完成
        }

        // 上月----------------------------------------------------------
        Map<String, BigDecimal> countLastMonthsale = saleOrderMapper.countSalesYm(city, DateUtils.getqytsydyt(DateUtils.isThisTime(yearMonth)),DateUtils.getqytsydr(DateUtils.isThisTime(yearMonth)));
//		BigDecimal lastMonthsale = countLastMonthsale.get("sales");

        BigDecimal lastsaleNumber = BigDecimal.ZERO;
        if (null != countLastMonthsale && countLastMonthsale.containsKey("sales") && countLastMonthsale.get("sales") != null) {
            lastsaleNumber = countLastMonthsale.get("sales");// 上个月已完成
        }
        // 同比去年------------------------------------------------------
        Map<String, BigDecimal> countBeforeYearsale = saleOrderMapper.countSalesYm(city, DateUtils.getqytqndrq(DateUtils.isThisTime(yearMonth)),DateUtils.getqytqn(DateUtils.isThisTime(yearMonth)));
        BigDecimal beforeYearsale = countBeforeYearsale == null ? BigDecimal.ZERO
                : countBeforeYearsale.get("sales");
        BigDecimal beforeYearsaleNumber = BigDecimal.ZERO;
        if (null != beforeYearsale) {
            beforeYearsaleNumber = beforeYearsale;// 上个月已完成
        }

        BigDecimal aa = BigDecimal.ZERO;
        if (saleNumber != null && totalTargetNumber != null && saleNumber.intValue() != 0
                && totalTargetNumber.intValue() != 0) {
            aa = saleNumber.divide(totalTargetNumber, 4, BigDecimal.ROUND_HALF_UP).multiply(new BigDecimal(100));
        }

        BigDecimal lastMonthGMV = BigDecimal.ZERO;
        if (null != lastMonthGMV && null != lastsaleNumber && lastMonthGMV.intValue() != 0
                && lastsaleNumber.intValue() != 0) {
            lastMonthGMV = saleNumber.subtract(lastsaleNumber);
        }

        BigDecimal bb = BigDecimal.ZERO;
        if (null != saleNumber && saleNumber.intValue() != 0 && null != lastsaleNumber
                && lastsaleNumber.intValue() != 0) {
            bb = (saleNumber.subtract(lastsaleNumber)).divide(lastsaleNumber, 4, BigDecimal.ROUND_HALF_UP)
                    .multiply(new BigDecimal(100));
        }
        BigDecimal cc = BigDecimal.ZERO;
        if (saleNumber.intValue() != 0 && beforeYearsaleNumber.intValue() != 0) {
            cc = (saleNumber.subtract(beforeYearsaleNumber)).divide(beforeYearsaleNumber, 4, BigDecimal.ROUND_HALF_UP)
                    .multiply(new BigDecimal(100));
        }

        BigDecimal tk = countSalesYm == null ? BigDecimal.ZERO : countSalesYm.get("tk");
        tk = tk == null ? BigDecimal.ZERO : tranMoney2(tk);

        BigDecimal zf = countSalesYm == null ? BigDecimal.ZERO : countSalesYm.get("zf");
        BigDecimal tkl = BigDecimal.ZERO;
        if (tk.intValue() != 0 || zf.intValue() != 0) {
            tkl = tk.multiply(new BigDecimal(10000)).divide(zf, 4, BigDecimal.ROUND_HALF_UP).multiply(new BigDecimal(100));// 总退款除总支付
        }

        Map<String, Object> data = new HashMap<String, Object>();
        data.put("GMV", tranMoney2(saleNumber));// 当月累计销售GMV
        data.put("currentMonthTarget", setScale(aa) + "%");//目标进度 当月已完成总目标 当月已完成除以当年总目标
        data.put("lastMonthGMV", tranMoney2(saleNumber.subtract(lastsaleNumber)));// 环比上月gmv 这个月-上个月=400万，
        data.put("lastMonthGRatio", setScale(bb) + "%");// 环比上月gmv比率 400万/上个月
        data.put("beforeYearGMV", tranMoney2(saleNumber.subtract(beforeYearsaleNumber)));//  同比去年gmv
        data.put("beforeYearGRatio", setScale(cc) + "%");// 同比去年gmv比率
        data.put("currMonthRefund", tk);// 当月销售退款多少万
        data.put("currMonthRefundRatio", setScale(tkl) + "%");// 当月销售退款率百分比

        System.out.println("第1个耗时:" + (System.currentTimeMillis() - start));
        return data;
    }

    private Map<String, Object> getServiceGMVData(String yearMonth, String city) {
        long start = System.currentTimeMillis();
//        String[] ym = yearMonth.split("-");
//        int year = Integer.parseInt(ym[0]);
//        int month = Integer.parseInt(ym[1]);
//        String months = month < 10 ? ("0" + month) : month + "";

        // 当年服务---------------------------------------------------------
//        int curryear = Calendar.getInstance().get(Calendar.YEAR);
//        String startMonth = curryear + "-01";
//        String endMonth = curryear + "-12";
        BigDecimal totalTargetNumber = saleOrderMapper.salesGmvTarget(city, null, "服务GMV", yearMonth, yearMonth);// startMonth,
        // endMonth

        // 当月服务---------------------------------------------------------
        BigDecimal sale = saleOrderMapper.countServiceYm(city, DateUtils.qytdyddyt(DateUtils.isThisTime(yearMonth)), DateUtils.getFrontDay(DateUtils.isThisTime(yearMonth)));
        BigDecimal saleNumber = BigDecimal.ZERO;
        if (sale != null) {
            saleNumber = sale;// 当月已完成
        }

        // 上月服务----------------------------------------------------------
        BigDecimal lastMonthsale = saleOrderMapper.countServiceYm(city,  DateUtils.getqytsydyt(DateUtils.isThisTime(yearMonth)),DateUtils.getqytsydr(DateUtils.isThisTime(yearMonth)));
        BigDecimal lastsaleNumber = BigDecimal.ZERO;
        if (null != lastMonthsale) {
            lastsaleNumber = lastMonthsale;// 上个月已完成
        }
        // 同比去年------------------------------------------------------
        BigDecimal beforeYearsale = saleOrderMapper.countServiceYm(city, DateUtils.getqytqndrq(DateUtils.isThisTime(yearMonth)),DateUtils.getqytqn(DateUtils.isThisTime(yearMonth)));
        BigDecimal beforeYearsaleNumber = BigDecimal.ZERO;
        if (null != beforeYearsale) {
            beforeYearsaleNumber = beforeYearsale;// 上个月已完成
        }


        BigDecimal aa = BigDecimal.ZERO;
        if (saleNumber != null && saleNumber.intValue() != 0 && totalTargetNumber != null
                && totalTargetNumber.intValue() != 0) {
            aa = saleNumber.divide(totalTargetNumber, 4, BigDecimal.ROUND_HALF_UP).multiply(new BigDecimal(100));
        }

        BigDecimal lastMonthGMV = setScale(saleNumber.subtract(lastsaleNumber));
        BigDecimal bb = BigDecimal.ZERO;
        if (lastsaleNumber != null && lastsaleNumber.intValue() != 0) {
            bb = setScale((saleNumber.subtract(lastsaleNumber)).divide(lastsaleNumber, 4, BigDecimal.ROUND_HALF_UP)
                    .multiply(new BigDecimal(100)));
            //bb = bb.intValue() > 0 ? bb : bb.multiply(new BigDecimal(-1));
        }

        BigDecimal cc = BigDecimal.ZERO;
        if (null != saleNumber && saleNumber.intValue() != 0 && null != beforeYearsaleNumber
                && beforeYearsaleNumber.intValue() != 0) {
            cc = setScale((saleNumber.subtract(beforeYearsaleNumber))
                    .divide(beforeYearsaleNumber, 4, BigDecimal.ROUND_HALF_UP).multiply(new BigDecimal(100)));
            cc = bb.intValue() > 0 ? cc : cc.multiply(new BigDecimal(-1));
        }

        List<String> mhlist = new ArrayList<String>();
        mhlist.add(yearMonth);
        List<Map<String, Object>> serviceGmvdanliangMonth = saleOrderMapper.serviceGmvdanliangMonth(city, "保洁,品类",
                mhlist);
        BigDecimal dl = BigDecimal.ZERO;
        if (null != serviceGmvdanliangMonth && serviceGmvdanliangMonth.size() > 0) {
            for (Map<String, Object> f : serviceGmvdanliangMonth) {
                if (f.containsKey("zdl")) {
                    dl = dl.add(new BigDecimal(f.get("zdl").toString()));
                }
            }
            //Map<String, Object> map = serviceGmvdanliangMonth.get(0);
            //dl = f.get("zdl");
        }

        Map<String, Object> data = new HashMap<String, Object>();
        data.put("GMV", tranMoney2(saleNumber));// 当月累计销售GMV
        data.put("currentMonthTarget", setScale(aa) + "%");// 当月目标进度 当月已完成总目标 当月已完成除以当年总目标
        data.put("lastMonthGMV", tranMoney2(lastMonthGMV));// 环比上月gmv 这个月-上个月=400万，
        data.put("lastMonthGRatio", bb + "%");// 环比上月gmv比率 400万/上个月
        data.put("beforeYearGMV", tranMoney2(saleNumber.subtract(beforeYearsaleNumber)));// 对比去年gmv
        data.put("beforeYearGRatio", cc + "%");// 对比去年gmv比率
        data.put("currentMonthOrderNumber", dl);// 当月服务单量
//		data.put("refund", saleOrderMapper.countRefundYm(yearMonth));// 当月退款金额

        System.out.println("第2个耗时:" + (System.currentTimeMillis() - start));
        return data;
    }

    public BigDecimal setScale(BigDecimal val) {
        return val.setScale(2, BigDecimal.ROUND_DOWN);
    }

    public String subLast(Double aa) {
        if (aa == null) {
            return null;
        }
        if (aa < 0) {
            aa = aa * -1;
        }
        try {
            return aa.toString().substring(0, aa.toString().indexOf(".") + 3);
        } catch (Exception e) {
            // TODO: handle exception
        }
        return aa.toString();
    }

    public Double subLast2(Double aa) {
        if (aa == null) {
            return null;
        }
        try {
            String s = aa.toString().substring(0, aa.toString().indexOf(".") + 3);
            return Double.valueOf(s);
        } catch (Exception e) {
            // TODO: handle exception
        }
        return aa;
    }

    public String getTypeValue(String businessTypeName) {
        List<String> list = new ArrayList<String>();
        if (businessTypeName.equals("保洁")) {
            list.add("保洁");
//        	list.add("普通保洁单次");
//        	list.add("普通保洁周期");
//        	list.add("鲸选保洁单次");
//        	list.add("鲸选保洁周期");
        }
        if (businessTypeName.equals("保姆")) {
            list.add("保姆");
        }
        if (businessTypeName.equals("品类")) {
            list.add("品类");
        }
        if (businessTypeName.equals("搬家")) {
            list.add("搬家");
        }
        if (businessTypeName.equals("附加")) {
            list.add("附加");
        }
        return StringUtils.join(list, ",");
    }

    public static void main(String[] args) {
        BigDecimal b = new BigDecimal("0.1037");
        System.out.println(b.doubleValue() > 0);
    }

	@Override
	public List<Map<String, Object>> getBjGoodsArrt() {
		// TODO Auto-generated method stub
		return saleOrderMapper.getBjGoodsArrt();
	}

}
