package com.jzlife.report.service.newreport.impl;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.jzlife.report.dao.hmorder.OnlineServiceSummaryMapper;
import com.jzlife.report.dao.manager.ServicePowerReportMapper;
import com.jzlife.report.dao.manager.ServiceQualityReportMapper;
import com.jzlife.report.dto.export.OnceFilingTicketExportDto;
import com.jzlife.report.excelExport.DayCustomerClaimDetailDTO;
import com.jzlife.report.excelExport.ReportComplaintmanagementDTO;
import com.jzlife.report.service.newreport.ServiceQualityReportService;
import com.jzlife.report.util.ResponseUtil;
import com.jzlife.report.util.page.BeanUtil;
import com.jzlife.report.vo.DayReportBaseVo;
import com.jzlife.report.vo.ReportBaseVo;
import com.jzlife.report.vo.ReportCustomerClaimAndPowerVo;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.DecimalFormat;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.time.temporal.ChronoUnit;
import java.time.temporal.TemporalAdjusters;
import java.util.*;
import java.util.concurrent.atomic.AtomicLong;
import java.util.stream.Collectors;
import java.util.stream.Stream;

@Service
@Slf4j
public class ServiceQualityReportServiceImpl implements ServiceQualityReportService {

    @Autowired
    private ServiceQualityReportMapper serviceQualityReportMapper;

    @Autowired
    private ServicePowerReportMapper servicePowerReportMapper;

    @Autowired
    private OnlineServiceSummaryMapper onlineServiceSummaryMapper;


    @Override
    public Map<String, Object> serviceQualityStackLine(ReportBaseVo vo) {
        LocalDate with = LocalDate.parse(vo.getEndTime() + "-01", DateTimeFormatter.ofPattern("yyyy-MM-dd"))
                .with(TemporalAdjusters.lastDayOfMonth());

        Map<String, Object> result = new HashMap<>(12);
        setCustomerComplaints(vo, result, with);

        /*
         *
         * 二期 保洁、品类 客诉一级、二级分类 按照时间筛选， 新增工单处理结果分布 、订单来源分布
         *
         */

        // 保洁客诉和品类分类
        setBJAndPLClaimData(vo, result);
        // 工单处理结果分布
        setBJAndPLClaimResultData(vo, with, result);

        // 订单来源分布
        List<Map<String, Object>> orderSource = serviceQualityReportMapper.selectOrderSourceDistribution(vo.getCity(),
                vo.getServiceStation(), vo.getStartTime() + "-01",
                with.format(DateTimeFormatter.ofPattern("yyyy-MM-dd")));
        result.put("orderSource", orderSource);
        // 保洁报备异常分类
        setExecptionData(vo, with, result);

        return result;
    }


    //设置客诉率及分布
    private void setCustomerComplaints(ReportBaseVo vo, Map<String, Object> result, LocalDate with) {
        List<Map<String, Object>> tempList = serviceQualityReportMapper.selectOrderAndClaimList(
                vo.getStartTime() + "-01", with.format(DateTimeFormatter.ofPattern("yyyy-MM-dd")), vo.getCity(),
                vo.getServiceStation());
        String monthFirstDay = LocalDate.now().with(TemporalAdjusters.firstDayOfMonth()).toString();
        String monthLastDay = LocalDate.now().with(TemporalAdjusters.lastDayOfMonth()).toString();
        List<Map<String, Object>> claimList = serviceQualityReportMapper.selectOrderAndClaimList(monthFirstDay,
                monthLastDay, vo.getCity(), vo.getServiceStation());
        //获取当月已完成工单及投诉信息
        setClaimInfo(claimList, result);
        List<String> allMonth = getBetweenMonth(vo.getStartTime(), vo.getEndTime());
        List<BigDecimal> totalRateList = new ArrayList<>();
        List<BigDecimal> bjRateList = new ArrayList<>();
        List<BigDecimal> plRateList = new ArrayList<>();
        //查询范围内 所有月份的投诉信息
        allMonth.forEach(month -> {
            AtomicLong totalOrder = new AtomicLong(0L);
            AtomicLong bjOrder = new AtomicLong(0L);
            AtomicLong plOrder = new AtomicLong(0L);
            AtomicLong totalClaim = new AtomicLong(0L);
            AtomicLong bjClaim = new AtomicLong(0L);
            AtomicLong plClaim = new AtomicLong(0L);
            tempList.forEach(map -> {
                String mapMonth = (String) map.get("month");
                if (StringUtils.equals(month, mapMonth)) {
                    totalOrder.set(totalOrder.get() + ((BigDecimal) map.get("total_order")).longValue());
                    bjOrder.set(bjOrder.get() + ((BigDecimal) map.get("bj_order")).longValue());
                    plOrder.set(plOrder.get() + ((BigDecimal) map.get("pl_order")).longValue());
                    totalClaim.set(totalClaim.get() + ((BigDecimal) map.get("total_claim")).longValue());
                    bjClaim.set(bjClaim.get() + ((BigDecimal) map.get("bj_claim")).longValue());
                    plClaim.set(plClaim.get() + ((BigDecimal) map.get("pl_claim")).longValue());
                }
            });
            BigDecimal totalRate = BigDecimal.ZERO;
            if (totalOrder.get() != 0) {
                totalRate = new BigDecimal(totalClaim.get()).multiply(new BigDecimal(100))
                        .divide(new BigDecimal(totalOrder.get()), 2, RoundingMode.HALF_UP);
            }
            totalRateList.add(totalRate);

            BigDecimal plRate = BigDecimal.ZERO;
            if (plOrder.get() != 0) {
                plRate = new BigDecimal(plClaim.get()).multiply(new BigDecimal(100))
                        .divide(new BigDecimal(plOrder.get()), 2, RoundingMode.HALF_UP);
            }
            plRateList.add(plRate);

            BigDecimal bjRate = BigDecimal.ZERO;
            if (bjOrder.get() != 0) {
                bjRate = new BigDecimal(bjClaim.get()).multiply(new BigDecimal(100))
                        .divide(new BigDecimal(bjOrder.get()), 2, RoundingMode.HALF_UP);
            }
            bjRateList.add(bjRate);

        });
        result.put("month", allMonth);
        result.put("series", Arrays.asList(totalRateList, bjRateList, plRateList));
    }

    private void setClaimInfo(List<Map<String, Object>> claimList, Map<String, Object> result) {
        DecimalFormat decimalFormat = new DecimalFormat("#0.00%");
        AtomicLong totalOrder = new AtomicLong(0L);
        AtomicLong bjOrder = new AtomicLong(0L);
        AtomicLong plOrder = new AtomicLong(0L);
        AtomicLong totalClaim = new AtomicLong(0L);
        AtomicLong bjClaim = new AtomicLong(0L);
        AtomicLong plClaim = new AtomicLong(0L);
        BigDecimal totalRate = BigDecimal.ZERO;
        BigDecimal plRate = BigDecimal.ZERO;
        BigDecimal bjRate = BigDecimal.ZERO;
        //其他服务单
        AtomicLong otherOrder = new AtomicLong(0L);
        AtomicLong otherClaim = new AtomicLong(0L);
        BigDecimal otherRate = BigDecimal.ZERO;

        for (Map<String, Object> map : claimList) {
            totalOrder.set(totalOrder.get() + ((BigDecimal) map.get("total_order")).longValue());
            bjOrder.set(bjOrder.get() + ((BigDecimal) map.get("bj_order")).longValue());
            plOrder.set(plOrder.get() + ((BigDecimal) map.get("pl_order")).longValue());
            totalClaim.set(totalClaim.get() + ((BigDecimal) map.get("total_claim")).longValue());
            bjClaim.set(bjClaim.get() + ((BigDecimal) map.get("bj_claim")).longValue());
            plClaim.set(plClaim.get() + ((BigDecimal) map.get("pl_claim")).longValue());

        }
        result.put("currentTotalOrder", totalOrder);
        result.put("currentBJOrder", bjOrder);
        result.put("currentPlOrder", plOrder);
        result.put("currentTotalClaim", totalClaim);
        result.put("currentBJClaim", bjClaim);
        result.put("currentPlClaim", plClaim);
        if (totalOrder.get() != 0) {
            totalRate = new BigDecimal(totalClaim.get()).divide(new BigDecimal(totalOrder.get()), 4,
                    RoundingMode.HALF_UP);
        }
        if (plOrder.get() != 0) {
            plRate = new BigDecimal(plClaim.get()).divide(new BigDecimal(plOrder.get()), 4, RoundingMode.HALF_UP);
        }
        if (bjOrder.get() != 0) {
            bjRate = new BigDecimal(bjClaim.get()).divide(new BigDecimal(bjOrder.get()), 4, RoundingMode.HALF_UP);
        }
        result.put("currentTotalRate", decimalFormat.format(totalRate));
        result.put("currentBJRate", decimalFormat.format(bjRate));
        result.put("currentPlRate", decimalFormat.format(plRate));

        result.put("currentBjClaim", bjClaim);
        result.put("currentPlClaim", plClaim);
        result.put("currentTotalClaim", totalClaim);
        //其他非保洁和品类的服务单和投诉
        otherOrder.set(new BigDecimal(totalOrder.get()).subtract(new BigDecimal(bjOrder.get())).subtract(new BigDecimal(plOrder.get())).longValue());
        otherClaim.set(new BigDecimal(totalClaim.get()).subtract(new BigDecimal(bjClaim.get())).subtract(new BigDecimal(plClaim.get())).longValue());
        if (otherOrder.get() != 0) {
            otherRate = new BigDecimal(otherClaim.get()).divide(new BigDecimal(otherOrder.get()), 4,
                    RoundingMode.HALF_UP);
        }

        result.put("currentOtherOrder", otherOrder);
        result.put("currentOtherClaim", otherClaim);
        result.put("currentOtherRate", decimalFormat.format(otherRate));

    }

    private void setExecptionData(ReportBaseVo vo, LocalDate with, Map<String, Object> result) {
        List<Map<String, Object>> exceptionReports = serviceQualityReportMapper.selectExceptionReport(vo.getCity(),
                vo.getServiceStation(), vo.getStartTime() + "-01",
                with.format(DateTimeFormatter.ofPattern("yyyy-MM-dd")));
        Map<String, Object> reportMap = new HashMap<>();
        List<List<Object>> reports = getStrings();
        List<Object> months = getStrings();
        months.add("month");
        List<String> initMonth = getBetweenMonth(vo.getStartTime(), vo.getEndTime());
        months.addAll(initMonth);
        reports.add(months);
        List<String> reportTypes = serviceQualityReportMapper.getExecptionReportType();
        for (String reportType : reportTypes) {
            List<Object> objects = new ArrayList<>();
            for (Object s : months) {
                String month = (String) s;
                if (month.equals("month")) {
                    objects.add(reportType);
                    continue;
                }
                Map<String, Object> monthMap = getMonthMap(month, reportType, exceptionReports);
                if (!monthMap.isEmpty()) {
                    objects.add(monthMap.get("sum"));
                } else {
                    objects.add(0);
                }
            }
            if (!objects.isEmpty()) {
                reports.add(objects);
            }
        }
        reportMap.put("exceptionReports", reports);
        result.put("exceptionReports", reportMap);
    }

    private Map<String, Object> getMonthMap(String month, String reportType,
                                            List<Map<String, Object>> exceptionReports) {
        for (Map<String, Object> objectMap : exceptionReports) {
            if (StringUtils.equals(month, (CharSequence) objectMap.get("month"))
                    && StringUtils.equals(reportType, (CharSequence) objectMap.get("type"))) {
                return objectMap;
            }
        }
        return new HashMap<>();

    }

    /* 工单处理结果 */
    private void setBJAndPLClaimResultData(ReportBaseVo vo, LocalDate with, Map<String, Object> result) {
        List<Map<String, Object>> ticketResult = serviceQualityReportMapper.selectTicketResult(vo.getCity(),
                vo.getServiceStation(), vo.getStartTime() + "-01",
                with.format(DateTimeFormatter.ofPattern("yyyy-MM-dd")));
        result.put("ticketResult", ticketResult);
    }

    /* 保洁和品类 客诉 和差评 */
    private void setBJAndPLClaimData(ReportBaseVo vo, Map<String, Object> result) {
        List<Map<String, Object>> bjClaimList = serviceQualityReportMapper.selectClaimList(vo.getStartTime(),
                vo.getEndTime(), vo.getCity(), vo.getServiceStation());
        List<String> bjOneClaim = getStrings();
        List<String> bjTwoClaim = getStrings();
        List<Long> bjOneCount = getStrings();
        List<Long> bjTwoCount = getStrings();
        List<String> plOneClaim = getStrings();
        List<String> plTwoClaim = getStrings();
        List<Long> plOneCount = getStrings();
        List<Long> plTwoCount = getStrings();
        bjClaimList.forEach(a -> {
            if (StringUtils.equals("1", (CharSequence) a.get("level"))
                    && StringUtils.equals("保洁", (CharSequence) a.get("service_type"))) {
                bjOneClaim.add((String) a.get("name"));
                bjOneCount.add((Long) a.get("total"));
            } else if (StringUtils.equals("1", (CharSequence) a.get("level"))
                    && StringUtils.equals("品类", (CharSequence) a.get("service_type"))) {
                plOneClaim.add((String) a.get("name"));
                plOneCount.add((Long) a.get("total"));
            } else if (StringUtils.equals("2", (CharSequence) a.get("level"))
                    && StringUtils.equals("保洁", (CharSequence) a.get("service_type"))) {
                bjTwoClaim.add((String) a.get("name"));
                bjTwoCount.add((Long) a.get("total"));
            } else if (StringUtils.equals("2", (CharSequence) a.get("level"))
                    && StringUtils.equals("品类", (CharSequence) a.get("service_type"))) {
                plTwoClaim.add((String) a.get("name"));
                plTwoCount.add((Long) a.get("total"));
            }

        });
        result.put("bjOneClaim", bjOneClaim);
        result.put("bjTwoClaim", bjTwoClaim);
        result.put("bjOneCount", bjOneCount);
        result.put("bjTwoCount", bjTwoCount);
        result.put("plOneClaim", plOneClaim);
        result.put("plTwoClaim", plTwoClaim);
        result.put("plOneCount", plOneCount);
        result.put("plTwoCount", plTwoCount);
    }

    private <T> ArrayList<T> getStrings() {
        return new ArrayList<>();
    }

    public List<String> getBetweenMonth(String start, String end) {
        List<String> list = getStrings();
        LocalDate startDate = LocalDate.parse(start + "-01");
        LocalDate endDate = LocalDate.parse(end + "-01");
        long distance = ChronoUnit.MONTHS.between(startDate, endDate);
        Stream.iterate(startDate, d -> d.plusMonths(1)).limit(distance + 1)
                .forEach(f -> list.add(f.toString().substring(0, 7)));
        return list;
    }

    @Override
    public Map<String, Object> claimSummaryAndPie() {
        Map<String, Object> objectMap = serviceQualityReportMapper.selectSomePoint();
        Map<String, Object> result = new HashMap<>();
        result.put("aliveDispose", objectMap.get("aliveTicket"));
        result.put("avgTime", objectMap.get("avgTime"));
        result.put("superTicketCount", objectMap.get("superTicketCount"));
        result.put("overTimeOrderCount", objectMap.get("overTimeTicketCount"));
        result.put("callInRate", objectMap.get("ibRate"));
        result.put("callOutRate", objectMap.get("obRate"));
        return result;
    }

    @Override
    public PageInfo<Map<String, Object>> dayCustomerClaimDetail(DayReportBaseVo vo) {
        PageHelper.startPage(vo.getStartPage(), vo.getLimit());
        return new PageInfo<>(serviceQualityReportMapper.dayCustomerClaimDetail(vo));

    }

    @Override
    public Object onlineChatSummary(Integer pageNo, Integer pageSize, String startDate,
                                    String endDate, String customerId, String userName, Boolean isExport) {
//		onlineServiceSummaryMapper.updateContent();
        if (isExport) {
            return onlineServiceSummaryMapper.selectOnlineChatSummary(startDate, endDate, customerId, userName);
        }

        PageHelper.startPage(pageNo, pageSize);
        return new PageInfo<>(
                onlineServiceSummaryMapper.selectOnlineChatSummary(startDate, endDate, customerId, userName));
    }


    /**
     * 报表三期 新增 客诉数 和服务能力 投诉 (不区分投诉是否成立)
     */
    @Override
    public Map<String, Object> customerClaimAndServicePowerLineGraph(ReportCustomerClaimAndPowerVo vo) {

        Map<String, Object> result = new HashMap<>(2);

        if (vo.getTimeType() == 1) {
            // 按月
            setCustomerClaimAndServerPowerByMonth(vo.getCity(), vo.getServiceStation(), vo.getStartMonth(),
                    vo.getEndMonth(), result);
            return result;
        } else if (vo.getTimeType() == 2) {
            // 按天
            setCustomerClaimAndServerPower(vo, result);
            return result;
        } else {
            return new HashMap<>();
        }
    }

    // 设置 客诉数 和服务能力 按月 汇总
    private void setCustomerClaimAndServerPowerByMonth(String city, String serviceStation, String startMonth,
                                                       String endMonth, Map<String, Object> result) {
        List<Map<String, Object>> tempList = servicePowerReportMapper.selectCustomerClaimAndServerPowerByMonth(city,
                serviceStation, startMonth, endMonth);
        Map<String, Object> customerClaimAndServerPower = new HashMap<>();
        List<String> everyMonth = tempList.stream().map(a -> (String) a.get("buiness_time"))
                .collect(Collectors.toList());
        List<Long> customerClaimCount = tempList.stream().map(a -> (Long) a.get("customer_claim_count"))
                .collect(Collectors.toList());
        customerClaimAndServerPower.put("customerClaimCount", customerClaimCount);
        List<Long> servicePowerCount = tempList.stream().map(a -> (Long) a.get("service_power_count"))
                .collect(Collectors.toList());
        customerClaimAndServerPower.put("servicePowerCount", servicePowerCount);
        result.put("erveryMonth", everyMonth);
        result.put("claimAndServerPower", customerClaimAndServerPower);
    }

    private void setCustomerClaimAndServerPower(ReportCustomerClaimAndPowerVo vo, Map<String, Object> result) {
        List<Map<String, Object>> tempList = servicePowerReportMapper.selectCustomerClaimAndServerPower(vo.getCity(),
                vo.getServiceStation(), vo.getStartTime(), vo.getEndTime());
        Map<String, Object> customerClaimAndServerPower = new HashMap<>();
        List<Long> customerClaimCount = tempList.stream().map(a -> (Long) a.get("customer_claim_count"))
                .collect(Collectors.toList());
        customerClaimAndServerPower.put("customerClaimCount", customerClaimCount);
        List<Long> servicePowerCount = tempList.stream().map(a -> (Long) a.get("service_power_count"))
                .collect(Collectors.toList());
        customerClaimAndServerPower.put("servicePowerCount", servicePowerCount);
        result.put("claimAndServerPower", customerClaimAndServerPower);
        List<String> everyDay = tempList.stream().map(a -> (String) a.get("buiness_time")).collect(Collectors.toList());
        result.put("everyDay", everyDay);
    }

    @Override
    public Object onlineChathz(Integer pageNo, Integer pageSize, String startDate,
                               String endDate, String userName, Boolean isExport) {
        if (isExport) {
            return onlineServiceSummaryMapper.onlineChathz(startDate, endDate, userName);
        }
        PageHelper.startPage(pageNo, pageSize);
        return new PageInfo<>(
                onlineServiceSummaryMapper.onlineChathz(startDate, endDate, userName));
    }

    @Override
    public List<DayCustomerClaimDetailDTO> dayCustomerClaimDetailNoPage(DayReportBaseVo baseVo) {
        return serviceQualityReportMapper.dayCustomerClaimDetailNoPage(baseVo);
    }

    @Override
    public List<ReportComplaintmanagementDTO> complaintManagementExport(String city, String serviceStation, String serverName,
                                                                        String ticketSn, String hemaWorkOrder, String startTime, String endTime) {

	return	serviceQualityReportMapper.complaintManagementExport(city, serviceStation, serverName, ticketSn, hemaWorkOrder, startTime, endTime);
    }

    @Override
    public PageInfo<ReportComplaintmanagementDTO> complaintManagement(String city, String serviceStation, String serverName, String ticketSn, String hemaWorkOrder, String startTime, String endTime, String pageNo, String pageSize) {
        PageHelper.startPage(Integer.parseInt(pageNo),Integer.parseInt(pageSize));
        List<ReportComplaintmanagementDTO> list = serviceQualityReportMapper.complaintManagementExport(city, serviceStation, serverName, ticketSn, hemaWorkOrder, startTime, endTime);
        return new PageInfo<>(list);
    }


	@Override
	public Object onceFilingTicket(Integer pageNo, Integer pageSize, String startDate, String endDate, String city,
			Boolean isExport) {
		if(isExport) {
			return serviceQualityReportMapper.getOnceFilingTicket(startDate, endDate, city);
		}
		PageHelper.startPage(pageNo, pageSize);
		List<OnceFilingTicketExportDto> dataList = serviceQualityReportMapper.getOnceFilingTicket(startDate, endDate, city);
		
		return ResponseUtil.ok(BeanUtil.toPagedResult(dataList));
	}

}
