package com.jzlife.report.service.report;

import com.alibaba.excel.EasyExcel;
import com.jzlife.report.compent.EasyExcelListener;
import com.jzlife.report.constant.ImportExcelConstant;
import com.jzlife.report.dao.reports.ImportDataConfigMapper;
import com.jzlife.report.dao.reports.ImportDataRecordMapper;
import com.jzlife.report.entity.reports.ImportDataConfig;
import com.jzlife.report.entity.reports.ImportDataRecord;
import com.jzlife.report.util.DateUtil;
import com.jzlife.report.util.file.FolderFileScanner;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.File;
import java.io.FileInputStream;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 文件导入，处理逻辑
 *
 * @author 36952
 */
@Service
public class FileScannerReadExcelService {

    @Autowired
    ImportDataConfigMapper importDataConfigMapper;

    @Autowired
    ImportDataRecordMapper dataRecordMapper;

    // 文件读取前，修改前缀
    public static String READ_PREFIX = "read-";

    /**
     * 扫描文件，不读取文件
     *
     * @return
     * @throws Exception
     */
    public List<ImportDataRecord> fileScanner() {
        // 先扫描，记录路径名称等（过滤调read开头的），
        // 再读取处理（查询 扫描时间+10分钟），处理前，先改文件名称(read-时间戳-)，再处理数据

        List<ImportDataRecord> dataList = new ArrayList<ImportDataRecord>();

        List<ImportDataConfig> configList = importDataConfigMapper.getAll();
        if (configList == null) {
            return dataList;
        }

        for (ImportDataConfig dt : configList) {
            Map<String, String> fi = new HashMap<String, String>();
            try {
                fi = FolderFileScanner.scanFiles(dt.getScanPath());
            } catch (Exception e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }

            if (fi.size() == 0) {
                return null;
            }

            fi.forEach((path, fileName) -> {
                if (!fileName.contains(READ_PREFIX) && (fileName.endsWith("xls") || fileName.endsWith("xlsx") || fileName.endsWith("csv"))) {
                    ImportDataRecord dr = new ImportDataRecord();
                    dr.setImportId(dt.getId());
                    dr.setTypeCode(dt.getTypeCode());
                    dr.setFileName(fileName);
                    dr.setFilePath(path);
                    dr.setCreateTime(new Date());
                    dr.setState(ImportExcelConstant.STATE_SCANNER);
                    dataList.add(dr);
                    dataRecordMapper.insertSelective(dr);
                }
            });
        }

        return dataList;
    }

    /**
     * 修改文件名称，再读取文件,返回修改后的路径，直接作为poi读取的路径
     */
    public List<ImportDataRecord> readFileAndReName() {
        // 五分钟前的文件
        List<ImportDataRecord> list = dataRecordMapper.selectByStateCreateTime(ImportExcelConstant.STATE_SCANNER,
                DateUtil.addMinute(-5));

        if (list == null) {
            return new ArrayList<ImportDataRecord>();
        }

        List<ImportDataRecord> tempList = new ArrayList<ImportDataRecord>();

        list.forEach(e -> {
            // System.currentTimeMillis() +
            String newName = e.getFileName().replace(e.getFileName(),
                    READ_PREFIX + "-" + e.getFileName());
            String newPathName = e.getFilePath().replace(e.getFileName(), newName);
            Boolean boo = FolderFileScanner.fileRename(e.getFilePath(), newPathName);

            if (boo) {
                long size = FolderFileScanner.getFileSize(newPathName);

                e.setFileSize(size);
                e.setReadTime(new Date());
                e.setState(ImportExcelConstant.STATE_READING);
                e.setFileName(newName);
                e.setFilePath(newPathName);
                dataRecordMapper.updateByPrimaryKeySelective(e);
                tempList.add(e);
            }

        });

        return tempList;
    }

    // state 0被扫描到，1读取处理中，2处理成功，3处理失败
    public void updateComplate(Integer id, Integer state, Integer readLine, String comment) {
        ImportDataRecord record = new ImportDataRecord();
        record.setId(id);
        record.setReadLine(readLine);
        if (state == 1) {
            record.setReadTime(new Date());
        }
        record.setComment(comment);
        record.setState(state);
        if (state == 2 || state == 3) {
            record.setCompleteTime(new Date());
        }

        dataRecordMapper.updateByPrimaryKeySelective(record);
    }

    ;

    /**
     * 删除15天前的文件
     */
    public void deleteFile() {
        //删除15天前的文件
        List<ImportDataRecord> list = dataRecordMapper.selectByStateCreateTime(ImportExcelConstant.STATE_SUCCESS,
                DateUtil.addDay(new Date(), -15));

        for (ImportDataRecord s : list) {
            FolderFileScanner.deleteFile(s.getFilePath());
            ImportDataRecord temp = new ImportDataRecord();
            temp.setId(s.getId());
            temp.setDeleted(true);
            temp.setDeletedTime(new Date());
            dataRecordMapper.updateByPrimaryKeySelective(temp);
        }
    }

    // 读取excel
    public void ReadExcel(String filePath, ImportDataRecord dr, Class<?> t) {
        // headRowNumber(1) 从哪一行开始读 因为这里涉及到表头 1就是从第一行开始读。
        try {
            File file = new File(filePath);
            EasyExcel.read(new FileInputStream(file), t, new EasyExcelListener(dr)).sheet().headRowNumber(1).doRead();
        } catch (Exception e) {
            e.printStackTrace();
        }

    }

    public List<ImportDataConfig> getDayScanProfiles() {
        return importDataConfigMapper.getDayScanProfiles();
    }

    public List<File> preProcessImport(String realFilePath, ImportDataConfig importDataConfig) {
        File dir = new File(realFilePath);
        if (!dir.isDirectory()) {
            System.out.println(realFilePath + "文件路径不是文件夹");
            return null;
        }
        File[] files = dir.listFiles();
        if (null == files || files.length == 0) {
            System.out.println(realFilePath + "文件夹为空");
            return null;
        }
        return Arrays.stream(files).filter(file -> {
            String fileName = file.getName();
            String key = importDataConfig.getKeyWord();
            boolean flag = true;
            if (key.contains(":")) {
                String[] strings = key.split(":");
                for (String string : strings) {
                    if (string.startsWith("!")) {
                        flag = !fileName.contains(string.substring(1));
                    } else {
                        if (!fileName.contains(string)) {
                            flag = false;
                            break;
                        }
                    }
                }
            } else {
                flag = fileName.contains(key);
            }
            return flag;
        }).collect(Collectors.toList());
    }
}
